SystemOrganization addCategory: #Time!
SystemOrganization addCategory: #'Time-Examples'!
SystemOrganization addCategory: #'Time-TimeZones'!
SystemOrganization addCategory: #'Time-TimeZones-tests'!
SystemOrganization addCategory: #'Time-UTC'!

Object subclass: #TimeZoneDstTransitionWatcher
	instanceVariableNames: 'lastTick transitionTick monitorProcess accessProtect'
	classVariableNames: 'DstMonitor'
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!TimeZoneDstTransitionWatcher commentStamp: 'dtl 10/31/2004 09:32' prior: 0!
Squeak uses a Time>>totalSeconds clock that jumps forward or backward at daylight savings time transitions. During the one hour period following a "fall back" transition, the calculation of DateAndTime>>now is ambiguous.

This class provides a monitor to keep track of any recent "fall back" transitions, enabling the correct DateAndTime to be chosen for any value of the Squeak seconds clock. The #fallBackOccurredDuringHourBefore: method provides a test to determine if the fall back transition has occurred withing the last hour.

The monitor process runs continuously, and therefore uses a certain amount of CPU that may be noticeable on small systems.

It is possible to provide a primitive for system time that answers both the current seconds clock and the current time zone offset. This primitive would make the time zone monitor unnecessary.!

!TimeZoneDstTransitionWatcher class methodsFor: 'instance creation' stamp: 'dtl 10/31/2004 09:10'!
clockWatcher

	"TimeZoneDstTransitionWatcher clockWatcher"

	^ DstMonitor ifNil: [DstMonitor := self new]! !

!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:36'!
initialize

	"self initialize"

	^ self startMonitoring! !

!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:16'!
startMonitoring

	"self startMonitoring"

	^ self clockWatcher! !

!TimeZoneDstTransitionWatcher class methodsFor: 'class initialization' stamp: 'dtl 10/31/2004 09:17'!
stopMonitoring

	"self stopMonitoring"

	DstMonitor ifNotNilDo: [:m |
		m stop.
		DstMonitor := nil.
		^ m].
	^ nil! !

!TimeZoneDstTransitionWatcher class methodsFor: 'tests' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
verifyDstTransition
	"Log time info during the Fall 2004 DST transition. Set the operating system
	clock to 00:58am on Sunday October 31, 2004 (or some other convenient fall
	back transition). Then run this method (three hours run time).
	Look that the DST.log file to verify that the clock was set back as expected.
	Note that it is not possible to do this test without actually setting the operating
	system clock, since we want to ensure that Squeak is tracking the actual
	behavior of the operating system."

	"[self verifyDstTransition] forkAt: Processor userBackgroundPriority"

	| fileName delay f t dt str fallTransitionTime shortDelay endTime now |
	fileName := 'DST.log'.
	fallTransitionTime := 1099202422. "Fall 2004 in Detroit"

	"Match time zone to the OS time zone setting"
	TimeZoneDatabase setLocalTimeZone: 'right/America/Detroit'.

	delay := Delay forMilliseconds: 9600. "Just under 10 seconds"
	shortDelay := Delay forMilliseconds: 1000.
	[f := FileStream fileNamed: fileName.
	"Wait for a 10 second boundary"
	[(Time totalSeconds \\ 10) = 0] whileFalse: [(Delay forMilliseconds: 100) wait].

	"Run for three hours (six times per minute)"
	endTime := PointInTime now asAbsoluteSeconds + (3600 * 3).
	[(now := PointInTime now asAbsoluteSeconds) < endTime] whileTrue:
		[t := Time totalSeconds.
		dt := DateAndTime now.
		str := 'Squeak seconds: ', t printString, ', DateAndTime: ', dt printString.
		f nextPutAll: str; nextPut: Character lf; flush.
		((now - fallTransitionTime) abs < 21)
			ifTrue:
				[shortDelay wait]
			ifFalse:
				[delay wait.
				[(Time totalSeconds \\ 10) = 0] whileFalse: [(Delay forMilliseconds: 100) wait]]].
	] ensure: [f close]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 11/21/2000 18:59'!
accessProtect

	^ accessProtect ifNil: [accessProtect := Semaphore forMutualExclusion]! !

!TimeZoneDstTransitionWatcher methodsFor: 'testing' stamp: 'dtl 11/22/2000 09:46'!
fallBackOccurredDuringHourBefore: aLocalSecondsCount
	"Answer true if a daylight savings time fall back transition has occured
	within the last hour prior to aLocalSecondsCount, which is assumed to
	represent the time now."

	"TimeZoneDstTransitionWatcher clockWatcher fallBackOccurredDuringHourBefore: Time totalSeconds"

	^ self transitionTick notNil and:
		[(aLocalSecondsCount - transitionTick) < 3600]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'dtl 11/13/2004 13:34'!
initialize

	DateAndTime addDependent: self.
	TimeZoneDatabase addDependent: self.
	TimeZoneDatabase systemDatabase addDependent: self.
	self start! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 10/31/2004 09:12'!
isFallBack: secondsCount
	"True if secondsCount is between 0 and 3600 seconds less than the previously
	observed seconds clock tick, non-inclusive. This situation occurs when the
	clock is set back at a fall daylight savings time transition."

	| diff |
	self lastTick ifNil: [^ false].
	diff := secondsCount - self lastTick.
	^ (diff < 0) and: [diff > -3600]! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:28'!
lastDstTransitionBefore: aPointInTime

	"self new lastDstTransitionBefore: PointInTime now"

	DateAndTime localTimeZone dstTransitionTimes
		inject: nil
		into: [:last :pt |
			(aPointInTime <= pt) ifTrue: [^ last].
			pt].
	^ nil
! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
lastTick
	"Answer the value of lastTick"

	^ lastTick! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
lastTick: anObject
	"Set the value of lastTick"

	lastTick := anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 09:05'!
monitorProcess
	"Answer the value of monitorProcess"

	^ monitorProcess! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
monitorProcess: anObject
	"Set the value of monitorProcess"

	monitorProcess := anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 11/23/2004 11:46'!
monitorProcessBlock
	"The monitor process needs to wake up only occasionally to keep the last
	tick reasonably up to date. PointInTime>>now calls #updateLastTick: with the
	current Smalltalk seconds clock in order to force an immediate update. This
	ensures that the DST monitor is up to date as of the call to PointInTime>>now,
	as required for a correct result at times very close to DST transition times."

	| delay |
	delay := Delay forMilliseconds: 3000.
	^ [[self updateLastTick: Time totalSeconds.
		delay wait] repeat]! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
probableLastTransitionTick
	"If possible, answer the last Smalltalk seconds DST transition for the current
	time zone."

	"self new probableLastTransitionTick"

	^ self probableLastTransitionTickBefore: PointInTime now

! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'PaulDeBruicker 2/7/2019 16:53'!
probableLastTransitionTickBefore: aPointInTime
	"If possible, answer the last Smalltalk seconds DST transition before aPointInTime
	for the current time zone. This method permits unit tests to trick the DST transition
	watcher into working for a different time setting than the current system time."

	"self new probableLastTransitionTickBefore: PointInTime now"

	^ (self lastDstTransitionBefore: aPointInTime)
		ifNotNil: [:pt | pt asLocalSmalltalkSeconds]
! !

!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'dtl 10/31/2004 09:07'!
start

	self stop.
	self monitorProcess:
		self monitorProcessBlock fork! !

!TimeZoneDstTransitionWatcher methodsFor: 'initialize-release' stamp: 'PaulDeBruicker 2/7/2019 08:52'!
stop

	self monitorProcess ifNotNil: [:p |
		p terminate.
		self monitorProcess: nil]! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
transitionTick
	"Answer the value of transitionTick"

	^ transitionTick! !

!TimeZoneDstTransitionWatcher methodsFor: 'accessing' stamp: 'dtl 10/31/2004 08:47'!
transitionTick: anObject
	"Set the value of transitionTick"

	transitionTick := anObject! !

!TimeZoneDstTransitionWatcher methodsFor: 'updating' stamp: 'dtl 11/13/2004 14:03'!
update: aParameter
	"If the system time zone setting has changed, then the last recorded
	transition tick is no longer valid."

	aParameter == #timeZoneForDateAndTime
		ifTrue: [self transitionTick: self probableLastTransitionTick].
	aParameter == #timeZoneDatabaseDefaultZone
		ifTrue: [self transitionTick: self probableLastTransitionTick].
	aParameter == #timeZoneDatabaseForSystem
		ifTrue: [self transitionTick: self probableLastTransitionTick.
			TimeZoneDatabase systemDatabase addDependent: self].! !

!TimeZoneDstTransitionWatcher methodsFor: 'monitor' stamp: 'dtl 11/22/2000 09:49'!
updateLastTick: currentLocalSmalltalkSeconds
	"Update the value of lastTick, and set transitionTick if a DST boundary
	has been crossed."

	self accessProtect critical:
		[(self isFallBack: currentLocalSmalltalkSeconds)
			ifTrue: [self transitionTick: currentLocalSmalltalkSeconds].
		self lastTick: currentLocalSmalltalkSeconds]
! !

Object subclass: #TimeZoneProxy
	instanceVariableNames: 'abbreviation transform'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!TimeZoneProxy commentStamp: 'dtl 11/14/2004 13:33' prior: 0!
TimeZoneProxy is a TimeZone that obtains its information from an instance of TimeZoneRuleSet. This may be the LocalTimeTransform for the local time zone on this system obtained from a TimeZoneDatabase; or it may be the TimeZoneDatabase itself, in which case the proxy refers to the currently active LocalTimeTransform for the database.

To set the system time zone to a proxy for the time zone database:
	TimeZoneProxy setDynamicTransform

To set the local time zone for Detroit using a time zone that understands leap seconds:
	TimeZoneDatabase setLocalTimeZone: 'right/America/Detroit'
Or for a Detroit time zone that ignores leap seconds:
	TimeZoneDatabase setLocalTimeZone: 'posix/America/Detroit'

When configured in this manner, the name of the current local time zone is:
	DateAndTime localTimeZone transform defaultTimeZone timeZoneName

The chain of references is as follows:
	DateAndTime class>>localTimeZone ==> aTimeZoneProxy
	 aTimeZoneProxy>>transform ==> aTimeZoneDatabase
	   aTimeZoneDatabase>>defaultTimeZone ==> aTimeZoneRuleSet

This configuration permits DateAndTime to use UTC offset, daylight savings adjustment, and leap second adjustment for any point in time. When a daylight savings time transition occurs or a leap second adjustment is made, dates and times will automatically be adjusted accordingly. If a different time zone is selected as the current time zone in the database, DateAndTime will automatically refer to the new setting.

To undo the above configuration and return DateAndTime to its default configuration:
	DateAndTime localTimeZone: TimeZone default

This default setting may be required for existing DateAndTime test cases to pass, because some of the tests are not aware of leap second adjustments.

Notes:
To find a suitable time zone for e.g. Detroit, use:
	TimeZoneDatabase grepFor: 'Detroit'
To set the a dynamic time zone proxy for a time zone in one step:
	TimeZoneProxy setDynamicTransform: 'posix/America/Detroit'
If you want to set the time zone abbreviation to a preferred value:
	TimeZoneProxy setDynamicTransform: 'posix/America/Detroit' abbreviation: 'EST'

!

!TimeZoneProxy class methodsFor: 'instance creation' stamp: 'dtl 11/10/2004 05:24'!
dynamicTransform
	"Answer a proxy that uses the system time zone database. This in turn
	will refer to the currently active local time transform in the database."

	"TimeZoneProxy dynamicTransform"

	^ (self basicNew transform: TimeZoneDatabase systemDatabase) initialize! !

!TimeZoneProxy class methodsFor: 'initialize-release' stamp: 'dtl 2/15/2007 22:17'!
initialize
	"Configure the system for a time zone proxy, and select UTC as the default time zone.
	The user is expected to set the correct default following installation."

	self setDynamicTransform.
	self setLocalTimeZone: 'UTC'
! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 11/8/2004 21:13'!
setDynamicLocalTransform: aTimeZoneRuleSet abbreviation: abbreviation
	"Set the local time zone to a proxy for the TimeZoneRuleSet that describes this
	time zone. The time zone offset value will be dynamically calculated for any
	point in time. This causes the time zone offset to be dynamically corrected for
	daylight savings time and other time zone anomolies.

	Using a time zone proxy causes methods such as DateAndTime class>>now and
	Time class>>now to run more slowly, which could be a concern in applications
	that do a very large number of system time queries."

	"TimeZoneProxy setDynamicLocalTransform:
			(TimeZoneDatabase systemDatabase timeZoneFor: 'right/America/Detroit')
		abbreviation: 'EST'"

	| proxy |
	proxy := self transform: aTimeZoneRuleSet abbreviation: abbreviation.
	DateAndTime localTimeZone: proxy.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/10/2004 05:22'!
setDynamicTransform
	"Set the proxy to use the system time zone database. When the default time
	zone for the database is changed, the change is automatically reflected in
	this proxy. The time zone abbreviation for the active time zone is selected
	from one of possibly many valid abbreviations for that time zone, and will
	not in general be an expected value.

	This configuration should be used in applications where the system time
	zone may be reset in the time zone database, and where the time zone
	abbreviation is relatively unimportant."

	"TimeZoneProxy setDynamicTransform"

	| proxy |
	proxy := self dynamicTransform.
	DateAndTime localTimeZone: proxy.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/14/2004 13:06'!
setDynamicTransform: timeZoneName
	"Set the proxy to use the system time zone database, and set the database
	to use timeZoneName as the default time zone."

	"TimeZoneProxy setDynamicTransform: '/right/America/Detroit'"

	| proxy |
	proxy := self setDynamicTransform.
	self setLocalTimeZone: timeZoneName.
	^ proxy
! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 11/14/2004 13:09'!
setDynamicTransform: timeZoneName abbreviation: abbreviation
	"Set the proxy to use the system time zone database, and set the database
	to use timeZoneName as the default time zone. Set the time zone abbreviation
	to one of the possibly several valid abbreviations for timeZoneName."

	"TimeZoneProxy setDynamicTransform: '/right/America/Detroit' abbreviation: 'EST'"

	^ (self setDynamicTransform: timeZoneName) abbreviation: abbreviation
! !

!TimeZoneProxy class methodsFor: 'set local time zone' stamp: 'dtl 2/25/2019 08:18'!
setLocalTimeZone: tzName
	"To find a time zone for Detroit:
		TimeZoneDatabase grepFor: 'Detroit'
	Then use this method to set the desired time zone."

	"TimeZoneProxy setLocalTimeZone: 'Europe/Moscow' "
	"TimeZoneProxy setLocalTimeZone: 'right/America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'posix/America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'America/Detroit' "
	"TimeZoneProxy setLocalTimeZone: 'UTC' "
	"TimeZoneProxy setLocalTimeZone: 'GMT' "
	"TimeZoneProxy setLocalTimeZone: 'Europe/London' "
	"TimeZoneProxy setLocalTimeZone: 'Africa/Johannesburg' "
	"TimeZoneProxy setLocalTimeZone: 'America/Los_Angeles' "

	TimeZoneDatabase setLocalTimeZone: tzName
! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 9/20/2004 19:40'!
setStaticLocalTransform: aTimeZoneRuleSet abbreviation: abbreviation
	"Set the local time zone to a TimeZone with an offset corresponding to the
	offset of the local time zone at the present time. Time zone offsets will not
	be dynamically calculated, and this TimeZone will not be valid across
	daylight savings time changes.
	
	Unless the speed of system time queries is a serious concern, the
	#setDynamicLocalTransform:abbreviation: method is preferred."

	"TimeZoneProxy setStaticLocalTransform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'EST'"

	| proxy tz |
	proxy := self transform: aTimeZoneRuleSet abbreviation: abbreviation.
	tz := TimeZone
		offset: proxy offset
		name: proxy name
		abbreviation: proxy abbreviation.
	DateAndTime localTimeZone: tz.
	^ proxy! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 11/10/2004 05:23'!
transform: aTimeZoneRuleSet
	"Answer a proxy that uses aTimeZoneRuleSet. The time zone abbreviation will be
	the first of possibly many valid abbreviations provided in the time zone
	rules."

	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone"

	^ (self basicNew transform: aTimeZoneRuleSet) initialize! !

!TimeZoneProxy class methodsFor: 'private' stamp: 'dtl 9/13/2004 12:59'!
transform: aTimeZoneRuleSet abbreviation: abbreviation
	"A time zone rule set typically has more than one abbreviation. This allows a
	specific abbreviation to be used. The abbreviation must be one of the abbreviations
	listed in the rule set."

	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'EST'"
	"TimeZoneProxy transform: TimeZoneDatabase systemDatabase defaultTimeZone abbreviation: 'XXX'"

	(aTimeZoneRuleSet tzNameAbbreviations includes: abbreviation asSymbol)
		ifTrue:
			[^ (self basicNew
				transform: aTimeZoneRuleSet;
				abbreviation: abbreviation asString) initialize]
		ifFalse:
			[self notify: abbreviation, ' not valid, must be one of ',
				aTimeZoneRuleSet tzNameAbbreviations asString]
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 13:17'!
abbreviation
	"Answer the value of abbreviation"

	^ abbreviation ifNil: [abbreviation := self transform abbreviation]! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 13:14'!
abbreviation: newAbbrev
	"Set the value of abbreviation"

	^ self
		abbreviation: newAbbrev
		ifNotValid: [self notify: '"', newAbbrev asString, '" is not a valid abbreviation for this time zone']
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 11/14/2004 12:30'!
abbreviation: anObject ifNotValid: errorBlock
	"Set the value of abbreviation. If the abbreviation is not valid for the current
	time zone, evaluate errorBlock."

	| newAbbrev possibleValues |
	newAbbrev := anObject asString.
	possibleValues := (self transform isNil
			ifTrue: [Array with: newAbbrev]
			ifFalse: [transform tzNameAbbreviations])
		collect: [:e | e asString].
	(possibleValues includes: newAbbrev)
		ifTrue: [abbreviation := newAbbrev]
		ifFalse: [errorBlock value]
! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'dtl 11/13/2004 12:20'!
dstTransitionTimes

	^ self transform dstTransitionTimes! !

!TimeZoneProxy methodsFor: 'initialize-release' stamp: 'dtl 11/14/2004 12:47'!
initialize

	self name; abbreviation.
	self transform addDependent: self.
	TimeZoneDatabase addDependent: self
! !

!TimeZoneProxy methodsFor: 'leap seconds' stamp: 'dtl 11/7/2004 20:42'!
leapSecondsAt: aPointInTime

	^ self transform leapSecondsAt: aPointInTime! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'dtl 11/23/2004 12:05'!
localOffsetSecondsAt: aPointInTime

	^ self transform localOffsetSecondsAt: aPointInTime! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'PaulDeBruicker 2/7/2019 08:53'!
name

	^ transform isNil
		ifTrue: [transform name]
		ifFalse: [self asString]! !

!TimeZoneProxy methodsFor: 'time zone offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
offset

	^ Duration seconds: (self transform localOffsetSecondsAt: PointInTime now)! !

!TimeZoneProxy methodsFor: 'printing' stamp: 'dtl 11/14/2004 13:17'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' for ', self transform name, ' (', self abbreviation asString, ')'
! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 9/13/2004 12:31'!
transform
	"Answer the value of transform"

	^ transform! !

!TimeZoneProxy methodsFor: 'accessing' stamp: 'dtl 9/13/2004 12:31'!
transform: anObject
	"Set the value of transform"

	transform := anObject! !

!TimeZoneProxy methodsFor: 'updating' stamp: 'dtl 8/2/2008 11:54'!
update: aParameter 
	"If a new system time zone database has been created, update my reference to it.
	If the default transform has changed, update my abbreviation if necessary."

	aParameter == #timeZoneDatabaseForSystem
		ifTrue: [(self transform isKindOf: TimeZoneDatabase)
				ifTrue:
					[transform removeDependent: self.
					self transform: TimeZoneDatabase systemDatabase.
					transform addDependent: self.
					abbreviation := transform abbreviation]].
	aParameter == #timeZoneDatabaseDefaultZone
		ifTrue: [self
			abbreviation: self abbreviation
			ifNotValid: [self abbreviation: self transform abbreviation ifNotValid: []]]! !

Object subclass: #WallClock
	instanceVariableNames: 'now timeZone date time'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-Examples'!

!WallClock commentStamp: '<historical>' prior: 0!
I am a demonstration of how to relate PointInTimeNow to Time and Date classes
by means of a TimeZone. Watch an instance of me in a Squeak Morphic inspector
(an MVC inspector will not be automatically updated, and is not very interesting
to watch).

I know how to watch the singleton instance of PointInTimeNow, and to update
the current date and time accordingly. Watch me in a Morphic inspector and
check if the time conversion methods are working correctly. My date and time
should be the same as those answered by Time class>>now and Date class>>today,
unless TimeZone>>here is a time zone with leap second compensation, in which
case my time will be offset by the number of leap seconds since 1970 (22 seconds
as of December 1999).!

!WallClock class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:32'!
for: aLocalTimeTransformName
	"Answer a WallClock in the specified time zone. Note that names beginning
	with 'right' include leap second tables, while names beginning with 'posix'
	do not. Select the type of time zone that matches the time setting for your
	computer's operating system."

	"WallClock for: 'right/America/Detroit'"
	"WallClock for: 'posix/America/Detroit'"
	"WallClock for: 'right/Europe/Berlin'"
	"WallClock for: 'right/Europe/Moscow'"

	| transform |
	transform := TimeZoneDatabase systemDatabase timeZoneFor: aLocalTimeTransformName.
	^ transform isNil
		ifTrue: [nil]
		ifFalse: [self forTimeZone: transform]
! !

!WallClock class methodsFor: 'instance creation' stamp: 'dtl 1/5/2000 19:56'!
forTimeZone: aLocalTimeTransform

	"WallClock forTimeZone: (LocalTimeTransform for: 'Europe/Berlin')"

	^ super new initialize timeZone: aLocalTimeTransform! !

!WallClock class methodsFor: 'instance creation' stamp: 'dtl 1/5/2000 19:56'!
new

	"WallClock new"

	^ self forTimeZone: (LocalTimeTransform here)! !

!WallClock methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
date

	^ date! !

!WallClock methodsFor: 'initialize-release' stamp: 'dtl 1/5/2000 19:53'!
initialize

	now := PointInTimeNow thisInstant.
	now addDependent: self! !

!WallClock methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
now

	^ now! !

!WallClock methodsFor: 'printing' stamp: 'dtl 1/5/2000 20:43'!
printOn: aStream

	| abs |
	now isNil
		ifTrue: [abs := nil]
		ifFalse: [abs := now absoluteTime].
	aStream nextPutAll: 'a '.
	abs isNil ifTrue: [aStream nextPutAll: 'stopped '].
	self class printOn: aStream.
	abs isNil ifFalse: [
		aStream nextPutAll: ' with date '.
		date printOn: aStream.
		aStream nextPutAll: ' and time '.
		time printOn: aStream.
		timeZone isNil ifFalse: [
			aStream nextPutAll: ' '.
			timeZone timeZoneName do: [:e | aStream nextPut: e]].
		aStream nextPutAll: ' from absoluteTime '.
		aStream nextPutAll: abs asInteger printString.
		aStream nextPutAll: ' ('.
		abs printOn: aStream.
		aStream nextPutAll: ')']! !

!WallClock methodsFor: 'initialize-release' stamp: 'dtl 12/30/1999 19:04'!
release

	now isNil ifFalse: [
		now removeDependent: self.
		now := nil]! !

!WallClock methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
time

	^ time! !

!WallClock methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
timeZone

	^ timeZone! !

!WallClock methodsFor: 'accessing' stamp: 'dtl 1/5/2000 19:48'!
timeZone: aLocalTimeTransform

	timeZone := aLocalTimeTransform! !

!WallClock methodsFor: 'updating' stamp: 'dtl 11/12/2004 17:00'!
update: aParameter

	| dt |
	now isNil ifFalse: [
		dt := self now asDateAndTimeArrayForTimeZone: self timeZone.
		date := dt at: 1.
		time := dt at: 2]
! !

!Date methodsFor: '*time-UTC' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
asDateInTimeZone: aTimeZone
	"The start time for a Date varies according to time zone, and various versions
	of Squeak handle Date instance creation differently. Answer a new instance
	with start time appropriate to aTimeZone."

	| approximateStartTime approximateOffset approximateOffsetSeconds actualOffsetSeconds actualStartTime |
	"Values are approximate because a DST transition might occur near this time"
	approximateStartTime := PointInTime
								fromSmalltalkSeconds: self asSeconds
								inTimeZone: aTimeZone.
	approximateOffsetSeconds := aTimeZone localOffsetSecondsAt: approximateStartTime.
	approximateOffset := Duration seconds: approximateOffsetSeconds.
	start := DateAndTime
				year: self year
				day: self day
				hour: 0
				minute: 0
				second: 0
				offset: approximateOffset.
	actualStartTime := PointInTime
							fromSmalltalkSeconds: start asSeconds
							inTimeZone: aTimeZone.
	actualOffsetSeconds := aTimeZone localOffsetSecondsAt: actualStartTime.
	actualOffsetSeconds = approximateOffsetSeconds ifFalse: [ | actualOffset |
		"Revise the calculation if a DST transition has affected the estimate"
		actualOffset := Duration seconds: actualOffsetSeconds.
		start := DateAndTime
					year: self year
					day: self day
					hour: 0
					minute: 0
					second: 0
					offset: actualOffset].
	^ self class starting: start
	

! !

!Timespan methodsFor: '*Time-UTC' stamp: 'PaulDeBruicker 3/13/2016 16:26'!
asDateTime
	^ start asDateTime! !

!Time class methodsFor: '*time-UTC' stamp: 'pad 3/10/2016 12:33'!
dateAndTimeFromLocalSeconds: secondCountSinceEpoch
	| dateAndTime |
	dateAndTime := DateAndTime fromLocalSeconds: secondCountSinceEpoch.
	^ Array with: dateAndTime asDate with: dateAndTime asTime! !

!Time methodsFor: '*time-UTC' stamp: 'PaulDeBruicker 3/3/2016 15:56'!
localSecondsSinceEpoch
	^self asDateAndTime localSecondsSinceEpoch! !

Magnitude subclass: #PointInTime
	instanceVariableNames: 'absoluteTime'
	classVariableNames: 'NumericRepresentationSelector Platform'
	poolDictionaries: ''
	category: 'Time-UTC'!

!PointInTime commentStamp: 'dtl 2/6/2012 20:26' prior: 0!
I represent a single point on an infinite time line. My origin is arbitrarily
chosen to be the first instant of the day of January 1, 1970, UTC. My unit
of measure is the second, which I may represent as a Float, a Double, or a
ScaledDecimal. My class method #numericRepresentationClass: controls the
actual data type of the numeric resolution. This is set during class initialization
but can be changed at any time.

I am based on the naive assumption that time can be represented as a single
continuum that may be experienced identically at different physical locations
and velocities, and such that two observers in two different locations can identify
a single PointInTime as a simultaneous event. These assumptions are not valid
when relativity is taken into account, but are acceptable for use in Newtonian
physics and Smalltalk.

A Float or Double has sufficient precision to represent a millisecond clock for
thousands of years, and can represent a microsecond clock for about the next
hundred years. This representation of time is used in order to emphasize the notion
of time as an infinite continuum, rather than a discreet integral value as implied
by computer hardware clocks. In other words, if there is to be a particle nature
to time, it should derive from physics and not from the peculiarities of computer
hardware.

In the event that a microsecond clock is required for Squeak in the
twenty-second century or later, the precision can be increased by shifting
the origin of time forward a couple hundred years (and adjusting conversion
methods accordingly), by changing to a higher precision numeric represention
(LargePositiveInteger or higher precision floating point data type), or by using
ScaledDecimal rather than a floating point representation.

Platform specific notes:

Squeak and Smalltalk/X use double precision floating point for class Float.
VisualWorks uses single precision float for Float, and double precision for
class Double. Therefore, the magnitude of a PointInTime is represented by
a Float or a Double, whichever provides the higher resolution.

On Squeak, access to the underlying high precision clock may be provided
by a pluggable primitive. See the methods in category primitives for examples,
and see the optional class TimePlugin for an implementation of the plugin.
Squeak has no built in concept of time zones or location-independent time
represention.

Smalltalk/X has an excellent implementation of time zones and time in
class AbsoluteTime. This should be used for any direct access to the
underlying system clock. For most practical applications, it also provides
excellent support for time zones using the underlying operating system
services.

VisualWorks provides access to the underlying operating system clock in
class Time, and some support for time zones in class TimeZone. The
implementation of TimeZone is portable across operating systems, but
is rather simplistic compared to Smalltalk/X.

Class PointInTime may be used in conjunction with a TimeZoneDatabase
to provide local time representation at any time and in any location
for which a TimeZoneRuleSet (time zone data set) is available. This
permits a platform independent implementation, but requires loading
time zone information into the Smalltalk image rather than accessing
it through operating system services.
!

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 11:58'!
absoluteSecondsAtPosixSeconds: posixSeconds inTimeZone: aTimeZone
	"Answer the count of absolute seconds corresponding to posixSeconds. If the local
	time zone does not account for leap seconds, then the result is posixSeconds.
	There is no direct mapping of Posix seconds to absolute seconds, because when
	a leap second is added to the calendar, there will be a Posix second that corresponds
	to two absolute seconds. For this reason an interative method is used to converge on
	the correct value.

	During a period of time after a leap second is inserted into the calendar, there
	may be two distinct values of PointInTime with the same local Posix seconds
	value. This method will answer only one of possibly two valid results.
	
	See also absoluteTimeAtSmalltalkSeconds:inTimeZone:, which has a similar problem
	with ambiguous conversion."

	"PointInTime absoluteSecondsAtPosixSeconds: 1098500000 inTimeZone: LocalTimeTransform here"

	| leap newTime |
	aTimeZone ifNil: [self notify: 'nil TimeZone'. ^ nil].
	(1 to: 10)
		inject: posixSeconds
		into: [:time :i |
			leap := aTimeZone leapSecondsAt: time.
			newTime := posixSeconds + leap.
			(newTime = time) ifTrue: [^ time].
			newTime].
	self error: 'algorithm does not converge'.
	^ nil! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/21/2000 19:00'!
absoluteSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone

	"PointInTime absoluteSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| possibleValues |
	possibleValues := self
		allAbsoluteSecondsCorrespondingTo: seconds
		inTimeZone: aTimeZone.
	(possibleValues size == 1)
		ifTrue: [^ possibleValues at: 1].
	(possibleValues size == 2)
		ifTrue:
			[(TimeZoneDstTransitionWatcher clockWatcher fallBackOccurredDuringHourBefore: seconds)
				ifFalse: [^ possibleValues at: 1]
				ifTrue: [^ possibleValues at: 2]].
	self error: 'expected either one or two valid absolute time values'
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 2/4/2012 17:44'!
absoluteSecondsNow
	"Answer the current time in absolute seconds"

	"PointInTime absoluteSecondsNow"

	^ self getAbsoluteSecondsFromLocalSecondsClock! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 12:02'!
absoluteTimeAtSmalltalkSeconds: smalltalkSeconds inTimeZone: aTimeZone
	"Find an absolute time for which the Smalltalk seconds clock is smalltalkSeconds.
	Answer an array with absolute seconds, UTC offset, and leap seconds for this
	time in aTimeZone.
	
	There is no direct mapping of local Smalltalk seconds to PointInTime, because
	time zone offset is a function of PointInTime rather than local Smalltalk time.
	For this reason an interative method is used to converge on a correct value.

	During a period of time after a daylight savings time transition, there may
	be two distinct values of PointInTime with the same local Smalltalk seconds
	value. This method will answer only one of possibly two valid results.
	
	See also absoluteSecondsAtPosixSeconds:inTimeZone:, which has a similar problem
	with ambiguous conversion."

	"PointInTime absoluteTimeAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| leap offset newTime |
	aTimeZone ifNil: [self notify: 'nil TimeZone'. ^ nil].
	(1 to: 10)
		inject: smalltalkSeconds - self posixOffset
		into: [:time :i |
			leap := aTimeZone leapSecondsAt: time.
			offset := aTimeZone localOffsetSecondsAt: time.
			newTime := smalltalkSeconds - self posixOffset - offset + leap.
			(newTime = time)
				ifTrue: [^ Array with: time with: offset with: leap].
			newTime].
	self error: 'algorithm does not converge'.
	^ nil! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
allAbsoluteSecondsCorrespondingTo: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone
	"Answer all values of absolute seconds that correspond to aLocalSmalltalkSecondsCount.
	Assume that the daylight savings time transition for the local time zone involves
	setting the clock forward or backward by exactly 3600 seconds (one hour). Assume
	that any leap second transitions involve setting the clock forward or back by
	one second."

	"self allAbsoluteSecondsCorrespondingTo: 3276637199
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276637200
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276640799
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteSecondsCorrespondingTo: 3276640800
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"

	| s |
	s := self firstAbsoluteSecondsAtSmalltalkSeconds: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone.
	^ (Array
		with: s - 3600
		with: s - 1
		with: s
		with: s + 1
		with: s + 3600)
			select: [:absoluteSecs |
				((PointInTime fromAbsoluteSeconds: absoluteSecs)
					asSmalltalkSecondsForTimeZone: aTimeZone) = aLocalSmalltalkSecondsCount]

! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
allAbsoluteTimesCorrespondingTo: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone
	"Answer all values of PointInTime that correspond to aLocalSmalltalkSecondsCount.
	Assume that the daylight savings time transition for the local time zone involves
	setting the clock forward or backward by exactly 3600 seconds (one hour). Assume
	that any leap second transitions involve setting the clock forward or back by
	one second."

	"self allAbsoluteTimesCorrespondingTo: 3276637199
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276637200
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276640799
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"
	"self allAbsoluteTimesCorrespondingTo: 3276640800
		inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')"

	| s |
	s := self firstAbsoluteSecondsAtSmalltalkSeconds: aLocalSmalltalkSecondsCount inTimeZone: aTimeZone.
	^ (Array
		with: s - 3600
		with: s - 1
		with: s
		with: s + 1
		with: s + 3600)
			select: [:absoluteSecs |
				((PointInTime fromAbsoluteSeconds: absoluteSecs)
					asSmalltalkSecondsForTimeZone: aTimeZone) = aLocalSmalltalkSecondsCount]
			thenCollect: [:abs | PointInTime fromAbsoluteSeconds: abs]
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
date: aDate time: aTime
	"Answer a PointInTime corresponding to the aDate and aTime in the
	local time zone."

	"PointInTime date: Date today time: Time now"

	^ self date: aDate time: aTime timeZone: LocalTimeTransform forSystem
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/4/2016 00:30'!
date: aDate time: aTime timeZone: aTimeZone
	"Answer a PointInTime corresponding to the aDate and aTime in the context
	of aTimeZone. Assume that aDate is intended to represent the local date in
	aTimeZone, regardless of its start value. This supports date values that may
	have been converted to UTC for purposes of date comparison, as is done in
	never versions of Squeak"

	"PointInTime date: Date today time: Time now timeZone: LocalTimeTransform here"

	| localDate |
	localDate := aDate asDateInTimeZone: aTimeZone.
	^ self fromSmalltalkSeconds: localDate localSecondsSinceEpoch  + aTime asSeconds inTimeZone: aTimeZone! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 10/31/2004 02:00'!
dateAndTimeArrayFromSmalltalkSeconds: seconds
	"This is from the Time class>>dateAndTimeFromSeconds method in Squeak,
	provided here for compability on other Smalltalk systems."

	"PointInTime dateAndTimeArrayFromSmalltalkSeconds: Time totalSeconds"

	| dt |
	dt := (self fromSmalltalkSeconds: seconds) asDateAndTime.
	^ Array
		with: dt asDate
		with: dt asTime! !

!PointInTime class methodsFor: 'general inquiries' stamp: ''!
dateAndTimeFromSmalltalkSeconds: seconds
	"This is from the Time class>>dateAndTimeFromSeconds method in Squeak,
	provided here for compability on other Smalltalk systems."

	^ Array
		with: (self dateFromSmalltalkSeconds: seconds)
		with: (Time fromSeconds: seconds \\ 86400)! !

!PointInTime class methodsFor: 'general inquiries' stamp: ''!
dateAndTimeNow
	"Answer the local representation of the current time as a Date and Time."

	"PointInTime dateAndTimeNow"

	^ self dateAndTimeFromSmalltalkSeconds: self now asLocalSmalltalkSeconds! !

!PointInTime class methodsFor: 'general inquiries' stamp: ''!
dateFromSmalltalkSeconds: seconds
	"Answer an instance of Date which is 'seconds' seconds after January 1,
	1901. This is a copy of the Date class>>fromSeconds method in Squeak,
	provided here for compability on other Smalltalk systems."

	^ Date fromDays: seconds // 86400! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/18/2004 19:20'!
firstAbsoluteSecondsAtSmalltalkSeconds: localSeconds inTimeZone: aTimeZone
	"Answer the first of the possibly two values of absolute seconds corresponding to localSeconds."

	"PointInTime firstAbsoluteSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ (self absoluteTimeAtSmalltalkSeconds: localSeconds inTimeZone: aTimeZone) first! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/21/2004 05:31'!
fromAbsoluteSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC"

	"PointInTime fromPosixSeconds: ((Date today asSeconds) + (Time now asSeconds) + 14400)"

	^ super new absoluteTime: seconds! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
fromPosixSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	"PointInTime fromPosixSeconds: ((Date today asSeconds) + (Time now asSeconds) + 14400)"

	^ self fromPosixSeconds: seconds inTimeZone: LocalTimeTransform forSystem! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/20/2004 12:00'!
fromPosixSeconds: seconds inTimeZone: aTimeZone
	"Answer aPointInTime at a number of seconds after 1-Jan-1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	^ super new absoluteTime: (self absoluteSecondsAtPosixSeconds: seconds inTimeZone: aTimeZone)! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 12:10'!
fromSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 in the local time zone."

	"PointInTime fromSeconds: Time totalSeconds"

	^ self fromSmalltalkSeconds: seconds
! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 11/12/2004 17:55'!
fromSmalltalkSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901."

	"PointInTime fromSmalltalkSeconds: Time totalSeconds"

	^ self fromSmalltalkSeconds: seconds inTimeZone: LocalTimeTransform forSystem! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 09:54'!
fromSmalltalkSeconds: seconds inTimeZone: aTimeZone
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 in a time zone.
	The time zone offset is calculated as of aPointInTime, which may not be the
	same offset as is currently in effect for aTimeZone. Both time zone offsets and
	leap seconds are used in the conversion.
	There is no direct mapping of local Smalltalk seconds to PointInTime, because
	time zone offset is a function of PointInTime rather than local Smalltalk time.
	For this reason an interative method is used to converge on the correct value."

	"PointInTime fromSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ self fromAbsoluteSeconds:
		(self absoluteSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone)! !

!PointInTime class methodsFor: 'instance creation' stamp: 'dtl 10/23/2004 13:33'!
fromUtcSmalltalkSeconds: seconds
	"Answer aPointInTime at a number of seconds after 1-Jan-1901 UTC, with no
	leap second compensation. This is equivalent to #fromSmalltalkSeconds: for a
	Smalltalk system located in Greenwich, England with its time zone set to a Posix
	time zone that ignores leap seconds."

	"PointInTime fromUtcSmalltalkSeconds: Time totalSeconds"

	^ super new absoluteTime: (seconds - self posixOffset)
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/21/2004 11:03'!
getAbsoluteSecondsForStx
	"This method is platform dependent, intended for Smalltalk/X only. Assume
	that the operating system provides UTC Posix seconds (this assumption may
	not be correct)."

	| osClass posixSeconds |
	osClass := Smalltalk at: #OperatingSystem.
	posixSeconds := osClass getOSTime.
	^ self absoluteSecondsAtPosixSeconds: posixSeconds inTimeZone: LocalTimeTransform here
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 6/27/2009 09:28'!
getAbsoluteSecondsFromLocalSecondsClock

	"PointInTime getAbsoluteSecondsFromLocalSecondsClock"

	| t |
	t := self totalLocalMilliseconds / 1000.
	"Force update to DST transition watcher, may be required if close to transition time."
	TimeZoneDstTransitionWatcher clockWatcher updateLastTick: t.
	^ self
		absoluteSecondsAtSmalltalkSeconds: t
		inTimeZone: DateAndTime localTimeZone
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/7/2011 22:49'!
getAbsoluteSecondsFromPluginOrLocalSecondsClock
	"Use a plugin if available, otherwise derive absolute seconds from
	the local seconds clock."

	"PointInTime getAbsoluteSecondsFromPluginOrLocalSecondsClock"

	| primTime |
	self primUtcWithOffset ifNotNilDo: [:a |
		^a first / 1000000].
	(primTime := self primPosixTimeMicrosecondResolution) isNil
		ifTrue:
			[^ self getAbsoluteSecondsFromLocalSecondsClock]
		ifFalse:
			[^ self
				absoluteSecondsAtPosixSeconds: primTime
				inTimeZone: DateAndTime localTimeZone]
! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 11/17/2004 06:26'!
getStxSeconds
        "This method is platform dependent, intended for Smalltalk/X only. Note
        that 'AbsoluteTime now asSeconds' produces correct results for Unix based
        Smalltalk/X. However, the asSeconds method is documented as strictly
        private, hence this (hopefully) safer implementation. An additional
        adjustment may be required to accommodate leap seconds."

        | classAbsoluteTime now |
        classAbsoluteTime := Smalltalk at: #AbsoluteTime.
        now := classAbsoluteTime now.
        ^ now - (classAbsoluteTime secondsSince1970: 0) + now utcOffset
! !

!PointInTime class methodsFor: 'initialize-release' stamp: 'dtl 3/2/2019 20:12'!
initialize
	self setNumericRepresentationSelector.
	self readme.! !

!PointInTime class methodsFor: 'instance creation' stamp: 'pad 3/14/2016 11:19'!
now
	"PointInTime now"

	^ self fromAbsoluteSeconds: self absoluteSecondsNow! !

!PointInTime class methodsFor: 'initialize-release' stamp: 'dtl 11/21/2004 08:37'!
numericRepresentationSelector: aClassSelector
	"This method is normally called during class initialization. A double precision
	floating point representation may be used (either Double or Float, depending
	on the Smalltalk platform). ScaledDecimal also may be used where availabile.
	To change the representation to an Integer (for example), inspect the following:
		PointInTime numericRepresentationSelector: #asInteger; now
	And to change back to normal for this platform, inspect this:
		PointInTime initialize; now"

	NumericRepresentationSelector := aClassSelector! !

!PointInTime class methodsFor: 'private' stamp: 'dtl 9/19/2004 23:50'!
platform
	"Answer what version of Smalltalk we are using."

	"PointInTime platform"

	^ Platform
! !

!PointInTime class methodsFor: 'private' stamp: 'pad 10/7/2011 09:29'!
platformSymbol
	"Answer what version of Smalltalk we are using."

	| versionString versionClass |
	(versionClass := Smalltalk at: #SystemVersion ifAbsent: []) isNil
		ifTrue: [versionString := Smalltalk version]
		ifFalse: [versionString := versionClass current version].
	('Squeak*' match: versionString) ifTrue: [^ #Squeak].
	('Pharo*' match: versionString) ifTrue: [^ #Pharo].
	('VisualWorks*' match: versionString) ifTrue: [^ #VisualWorks].
	('*Smalltalk.st*' match: versionString) ifTrue: [^ #'Smalltalk/X'].
	^ #unknown! !

!PointInTime class methodsFor: 'private' stamp: ''!
posixOffset
	"Logic is lifted from Ian's Unix support code.
	Squeak epoc	h is Jan 1, 1901.  Unix epoch is Jan 1, 1970: 17 leap years
     and 52 non-leap years later than Squeak."

	"52 * 365 + (17 * 366) * 24 * 60 * 60"

	^ 2177452800! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 13:23'!
posixSecondsAtSmalltalkSeconds: seconds
	"Answer the count of Posix seconds corresponding to local seconds in the local
	time zone."

	"PointInTime posixSecondsAtSmalltalkSeconds: Time totalSeconds"

	^ self posixSecondsAtSmalltalkSeconds: seconds inTimeZone: DateAndTime localTimeZone
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 11/20/2004 13:23'!
posixSecondsAtSmalltalkSeconds: seconds inTimeZone: aTimeZone
	"Answer the count of Posix seconds corresponding to local seconds in aTimeZone."

	"PointInTime posixSecondsAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	| abs |
	abs := self absoluteTimeAtSmalltalkSeconds: seconds inTimeZone: aTimeZone.
	^ abs first + abs second
! !

!PointInTime class methodsFor: 'primitives - deprecated' stamp: 'dtl 11/17/2004 19:50'!
primLocalSecondsClockWithOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	| result |
	<primitive: 'primitiveLocalSecondsClockWithOffset' module: 'TimeFunctions'>
	result := Array new: 2.
	result at: 1 put: Time primSecondsClock.
	result at: 2 put: 0.
	^ result! !

!PointInTime class methodsFor: 'primitives - deprecated' stamp: 'dtl 11/17/2004 19:51'!
primPosixTime
	"Number of seconds, expressed as a float, since since Jan 1, 1970 UTC."

	<primitive: 'primitivePosixTime' module: 'TimeFunctions'>
	^ nil! !

!PointInTime class methodsFor: 'primitives - deprecated' stamp: 'dtl 11/17/2004 19:51'!
primPosixTimeAndLocalOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	<primitive: 'primitivePosixTimeAndLocalOffset' module: 'TimeFunctions'>
	^ nil! !

!PointInTime class methodsFor: 'primitives - deprecated' stamp: 'dtl 11/17/2004 06:21'!
primPosixTimeMicrosecondResolution
	"Number of seconds, expressed as a float, since since Jan 1, 1970 UTC."

	<primitive: 'primitivePosixTimeMicrosecondResolution' module: 'TimeFunctions'>
	^ nil! !

!PointInTime class methodsFor: 'primitives' stamp: 'dtl 11/7/2011 23:02'!
primUtcWithOffset
	"Invoke the primitive implemented in recent Squeak VMs.
	Answer an array with UTC microseconds since the Posix epoch and
	the current seconds offset from GMT in the local time zone."

	"PointInTime primUtcWithOffset"

	<primitive: 'primitiveUtcWithOffset'>
	^nil! !

!PointInTime class methodsFor: 'documentation' stamp: 'dtl 3/2/2019 20:12'!
readme

	"PointInTime readme"

	StringHolder new acceptContents:
		'TimeZoneDatabase is a time zone database for Smalltalk. It answers the number of seconds offset from UTC for any time zone at any point in time in the range of the database rule set, as well as the number of leap seconds for a point in time (for tzfiles that contain the leap second rules).

TimeZoneProxy provides a dynamic time zone to calculate DateAndTime at any point in time in any time zone. Dynamic corrections for daylight savings time and leap seconds are made according to the local time zone setting. Time zone rules are provided by TimeZoneDatabase. A TimeZoneDstTransitionWatcher detects "fall back" daylight savings time transitions, and maintains correct conversions from the Squeak seconds clock to absolute time.

A full time zone database requires access to tzfile data files, typically distributed with Unix (Linux) systems, and often located in the /usr/share/zoneinfo directory.

When TimeZoneDatabase is initially installed, the system time zone (DateAndTime class>>localTimeZone) is set to ''UTC'' (no UTC offset, no daylight savings offset, no leap seconds). Here is how to set a new default time zone for Squeak:

1) Look for a suitable time zone to use. For example, if you live in Detroit, Michigan in the USA, you can find several valid time zone entries for Detroit by evaluating the following expression:
  TimeZoneDatabase grepFor: ''Detroit''

2) Set a time zone to use as the system default. For example, if you have decided to use ''America/Detroit'', tell Squeak to use this as its dynamic time zone setting:
  TimeZone setLocal: ''America/Detroit''

Time zones with names that begin with ''right'' include the rules for leap second adjustments, while time zones with names beginning with ''posix'' will ignore leap seconds. You may need to experiment with your system to find which time zone settings best match your requirements. If in doubt, use a time zone that does not include leap second tables.

The time zone database can be rebuilt or updated by reloading from files in the /usr/share/zoneinfo directory (a typical location for a Unix/Linux system). If your system does not have these files available, they may be copied from another system or obtained on the internet.
  TimeZoneDatabase buildSystemDatabaseForSqueak

If you want to stop using TimeZoneDatabase for your Squeak time zone settings and go back to a default TimeZone:
  DateAndTime localTimeZone: TimeZone default

For an explanation of the Squeak seconds clock and how it relates to various representations of time, see the "documentation" method category in class PointInTime.';
		openLabel: 'README'
! !

!PointInTime class methodsFor: 'initialize-release' stamp: 'PaulDeBruicker 6/5/2016 01:51'!
setNumericRepresentationSelector
	"Set the selector to use to provide a numeric representation with
	adequate precision. Some systems make a distinction between Float and
	Double, and others simply use a double precision implementation of Float
	for all cases. If available, ScaledDecimal may also be used."

	"PointInTime setNumericRepresentationSelector"

	Platform := self platformSymbol.
"	(Platform == #Squeak) ifTrue: [self numericRepresentationSelector: #asFloat]."
	(Platform == #Squeak) ifTrue: [self numericRepresentationSelector: #asScaledDecimal].
	(Platform == #VisualWorks) ifTrue: [self numericRepresentationSelector: #asDouble].
	(Platform == #'Smalltalk/X') ifTrue: [PointInTime numericRepresentationSelector: #asFloat].
	(Platform == #Pharo) ifTrue: [self numericRepresentationSelector: #asScaledDecimal].
	(Platform == #unknown) ifTrue: [self numericRepresentationSelector: #asFloat].! !

!PointInTime class methodsFor: 'documentation' stamp: 'dtl 11/9/2004 22:09'!
squeakSecondsClockInfo

	"(StringHolder new acceptContents: self squeakSecondsClockInfo; yourself)
		openLabel: 'Squeak seconds clock' "

	^
'The Squeak seconds clock represents the number of seconds since the Smalltalk epoch. The Smalltalk epoch is defined as 1 January 1901. The current value of this clock is available by evaluating Time class>>totalSeconds, which calls the ioSeconds() function in the Squeak virtual machine to obtain the current local time from the underlying operating system.

This is a wall-clock time, in the sense that the Squeak seconds clock corresponds to the setting of a clock on the wall located (geographically) in the current time zone. When a daylight savings time (DST) transition occurs, the hands on the wall clock are moved either forward or backward. Similarly, the Squeak seconds clock will jump forward or backward whenever a DST transition occurs. The current Time in Squeak will also jump forward and backward when DST transitions occur. Of course actual time proceeds forward relentlessly, but the display of time on the wall clock will have changed in order to make timekeeping more meaningful in the local time zone. Likewise, the displayed value of DateAndTime in Squeak will have suddenly adjusted itself to match the wall clock display.

In addition to DST transitions, leap seconds are occasionally added to or removed from the world calendar. In principle, the seconds hand on the wall clock is moved forward or back by one second whenever a leap second is defined. Likewise, the Squeak seconds clock is (or should be) adjusted forward or back when leap seconds occur.

It is also possible to define a notion of time as a magnitude, independent of all of the adjustments that are made to wall clocks in various locations around the world. Class PointInTime represents this universal clock, and uses the convention of describing a point in time as the total elapsed seconds since 1 January 1, 1970 in the UTC time zone. These are seconds as would be reported by a very accurate Caesium clock located in Greenwich, England. These seconds increase continuously without regard for geographic time zones, daylight savings time changes, or leap second adjustments used for local displays of time on wall clocks.

It is always possible to derive the value of the Squeak seconds clock from a PointInTime, given information about the local time zone offset and leap seconds that are in effect as of that PointIntTime. The TimeZoneDatabase provides this information. For any PointInTime and LocalTimeTransform (time zone), the local time zone offset and leap second count are known, and the Squeak seconds clock can be calculated.

However, the reverse transformation (Squeak seconds clock to PointInTime) is not possible in the general case. To understand why, consider the case of a Squeak system at 150 minutes (2 1/2 hours) after midnight on Sunday, 31 October, 2004 in an Eastern Standard Time time zone. This is a half hour after the "fall back" time zone change at the end of summer. Assuming that we have kept track of our local time zone and leap seconds, this occurs at the PointInTime 1099204222 seconds after 1 January, 1970. If we check the Squeak seconds clock at this point in time, it reports 3276639000 seconds.

This is all well and good, but exactly one hour earlier the DST transition had not yet happened. The PointInTime was 1099200622 seconds, and the Squeak seconds clock was 3276639000, exactly the same value as the Squeak seconds one hour later. Thus the Squeak seconds clock had the same value at two completely different points in time. Therefore there are periods of time during which it is not possible to determine which of two points in time is the correct PointInTime for the Squeak seconds clock.

It is of course possible to add a primitive to Squeak that makes the current PointInTime available directly. This, in conjunction with the information in the TimeZoneDatabase, provides enough information to do transformations from PointInTime to Squeak seconds and back without ambiguity. Alternatively, a primitive that reports current time (either UTC or local) along with current time zone offset provides the required information.

As a related note, operating systems may not report time zone offset correctly for points in time that occur outside of the current DST setting. But if TimeZoneDatabase is available in the Squeak image and a UTC seconds primitive is added, then Date and Time transformations can be reliably calculated for any PointInTime within the domain of the TimeZoneDatabase transforms.
'! !

!PointInTime class methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testFromSmalltalkSeconds
	"Answer 'OK' if PointInTime>>fromSmalltalkSeconds gives the right result, plus
	or minus one second to allow for clock updates while this test is running."

	"PointInTime testFromSmalltalkSeconds"

	| t1 t2 diff |
	t1 := Time totalSeconds.
	t2 := (PointInTime fromSmalltalkSeconds: t1 inTimeZone: LocalTimeTransform forSystem) asLocalSmalltalkSeconds.
	diff := t1 - t2.
	(diff abs < 1)
		ifTrue: [^ 'OK']
		ifFalse: [self notify: 'test failed'.
				^ 'Failed']! !

!PointInTime class methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testLocalSeconds
	"Answer 'OK' if PointInTime>>now>>asLocalSeconds gives the right result, plus
	or minus one second to allow for clock updates while this test is running.
	Put up a notifier if there is a discrepancy due to leap second handling.
	Time>>totalSeconds does not contain leap second compensation, but if the
	current time zone (TimeZone>>here) has leap second compensation, the value
	of PointInTime>>now>>asLocalSmalltalkSeconds will vary by the number of
	leap seconds. Both results can be considered valid, since leap seconds are
	commonly ignored."

	"PointInTime testLocalSeconds"

	| now currentSeconds diff pt t |
	pt := PointInTime now.
	t:=Time now.
	now := pt asLocalSmalltalkSeconds.
	currentSeconds := t asSeconds.
	diff := (currentSeconds - now) abs.
	now isNil
		ifFalse: [ 
			diff < 1
				ifTrue: [ ^ 'OK' ]
				ifFalse: [ 
					diff < 40
						ifTrue: [ 
							self notify: diff printString , ' leap second offset for this timezone versus OS timezone settings'.
							^ 'OK' ] ] ].
	self notify: 'test failed'.
	^ 'Failed'! !

!PointInTime class methodsFor: 'private-system clock' stamp: 'dtl 6/27/2009 09:28'!
totalLocalMilliseconds
	"Answer total local milliseconds as estimated by combining the results of the
	current total seconds clock with the value of the millisecond clock. Loop to
	avoid sampling at a millisecond clock rollover time."

	| t1 s t2 d millis |
	[t1 := Time primMillisecondClock.
	s := Time primSecondsClock.
	t2 := Time primMillisecondClock.
	d := t2 - t1.
	d < 0] whileTrue.
	millis := t2 rem: 1000.
	^ s * 1000 + millis
! !

!PointInTime class methodsFor: 'general inquiries' stamp: 'dtl 10/23/2004 09:52'!
tzOffsetAtSmalltalkSeconds: seconds inTimeZone: aTimeZone

	"PointInTime tzOffsetAtSmalltalkSeconds: Time totalSeconds inTimeZone: LocalTimeTransform here"

	^ (self absoluteTimeAtSmalltalkSeconds: seconds inTimeZone: aTimeZone) second! !

!PointInTime class methodsFor: 'documentation' stamp: 'dtl 11/9/2004 22:01'!
utcClockInfo

	"(StringHolder new acceptContents: self utcClockInfo; yourself)
		openLabel: 'UTC seconds clock' "

	^
'UTC is a measure of time corresponding to the angular rotation of the earth. As such, it is not a measure of time as measured by an atomic clock. UTC time is kept in synchronization with atomic time by the occasional insertion (or removal) of leap seconds in the UTC clock, although historically other methods of adjusting the clock (e.g. "elastic seconds") have been used, and the term "UTC" has had various definitions over the years. The distinction between rotational time (UTC) and atomic time can often be ignored. However, for purposes of physics and astronomy, they are different concepts and the small numerical differences between the two are significant.

Posix seconds are similar to UTC seconds, but are defined in such a way that leap seconds are ignored. The Posix seconds clock is thus effectively a "wall clock" representation of seconds, for which the Posix clock is adjusted forward or back when a leap second occurs. This is similar to the seconds clock in early Unix systems, which were designed prior to the invention of leap seconds.

Class PointInTime is intended to represent atomic time, such that time can be treated as a magnitude. Posix time varies from an atomic seconds clock as leap seconds are applied. For this reason, the conversion from PointInTime to and from Posix seconds should show a difference corresponding to the number of leap seconds that have been applied since 1 January, 1970.

If leap seconds are ignored, then Posix time and PointInTime seconds produce the same count of seconds. Many time zone tables ignore leap seconds, and many computer operating systems likewise ignore the issue. For example, the LocalTimeTransform for "right/America/Detroit" in a TimeZoneDatabase contains leap second rules, while the "posix/America/Detroit" transform always answers zero leap seconds. Both refer to the same geographical time zone, and for practical purposes the "posix/America/Detroit" transform is an appropriate choice to describe the time zone rules for Detroit, Michigan.

The LocalTimeTransforms in the TimeZoneDatabase contain transforms with or without leap second tables. The "posix/America/Detroit" transform ignores leap seconds, and is appropriate for a platform that is configured according to Posix specifications, while the "right/America/Detroit" transform accounts for leap seconds and provided a more realistic count of actual elapsed seconds since the Posix epoch.

It should not be assumed that the operatings system on which Squeak is hosted will handle leap seconds consistently. BSD Unix systems (including FreeBSD and Mac OSX) define the time_t system clock to represent atomic clock seconds and providing time2posix() and posix2time() functions for converting to and from Posix seconds. This allows correct counting of actual seconds since the Posix epoch. Other Unix and Linux systems may ignore leap seconds, and other operating systems may also handle leap seconds differently.

For purposes of converting Time and Date in Squeak to and from PointInTime seconds, we assume that Squeak will report its second clock as Posix seconds offset by the current time zone offset, regardless of the operating system and virtual machine on which it is running.
'! !

!PointInTime methodsFor: 'arithmetic' stamp: 'dtl 10/26/2004 20:48'!
- aPointInTime
	"Answer the difference in seconds between two points in time. The result
	may be converted to a Duration with #asDuration, but this method answers
	seconds in order to avoid loss of precision."

	^ absoluteTime - aPointInTime absoluteTime! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/26/2004 20:04'!
< aPointInTime

	^ (aPointInTime species == self species)
		and: [self absoluteTime < aPointInTime absoluteTime]! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/23/2004 13:16'!
= aPointInTime

	^ (aPointInTime species == self species)
		and: [aPointInTime absoluteTime = self absoluteTime]! !

!PointInTime methodsFor: 'accessing' stamp: ''!
absoluteTime

	^ absoluteTime! !

!PointInTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 6/5/2016 01:50'!
absoluteTime: aNumber
	NumericRepresentationSelector ifNil:[
		self class setNumericRepresentationSelector].
	absoluteTime := aNumber perform: NumericRepresentationSelector! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/23/2004 10:42'!
asAbsoluteSeconds
	"Answer the number of atomic clock seconds elapsed since 01-Jan-1970 UTC."

	^ self absoluteTime! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asDateAndTime
	"PointInTime now asDateAndTime"

	^ self asDateAndTimeForTimeZone: LocalTimeTransform forSystem
! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/23/2004 11:40'!
asDateAndTimeArrayForTimeZone: aTimeZone

	"PointInTime now asDateAndTimeArrayForTimeZone: LocalTimeTransform here"

	| dt |
	dt := self asDateAndTimeForTimeZone: aTimeZone.
	^ Array
		with: dt asDate
		with: dt asTime! !

!PointInTime methodsFor: 'converting' stamp: 'PaulDeBruicker 3/3/2016 15:54'!
asDateAndTimeForTimeZone: aTimeZone
	"PointInTime now asDateAndTimeForTimeZone: LocalTimeTransform here"

	| s offsetSeconds |
	s := self asSmalltalkSecondsForTimeZone: aTimeZone.
	offsetSeconds := aTimeZone localOffsetSecondsAt: self.
	s isNil
		ifTrue: [ ^ nil ]
		ifFalse: [ ^ DateAndTime fromSeconds: s offsetSeconds: offsetSeconds ]! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 3/10/2007 13:51'!
asDateAndTimeForTimeZoneNamed: tzName 

	"PointInTime now asDateAndTimeForTimeZoneNamed: 'CET' "

	(TimeZoneDatabase systemDatabase timeZoneFor: tzName)
		ifNotNilDo: [:tz | ^ (self asDateAndTimeForTimeZone: tz) asDateAndTime].
	self notify: 'no time zone entry for ', tzName asString.
	^ self asDateAndTime
! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 9/21/2004 06:49'!
asFloat

	^ absoluteTime asFloat! !

!PointInTime methodsFor: 'converting' stamp: ''!
asInteger

	^ absoluteTime asInteger! !

!PointInTime methodsFor: 'converting' stamp: ''!
asLocalDateAndTime
	"PointInTime now asLocalDateAndTime"

	^ self class dateAndTimeFromSmalltalkSeconds: self asLocalSmalltalkSeconds! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asLocalSmalltalkSeconds
	"Answer the number of seconds since January 1, 1901 in the local time zone. This
	number is suitable for creating instances of Time and Date."

	^ self asSmalltalkSecondsForTimeZone: LocalTimeTransform forSystem! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/27/2004 20:01'!
asNumber

	^ absoluteTime! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 11/12/2004 17:54'!
asPosixSeconds
	"Answer the number of Posix seconds since 01 January, 1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	"Array with: PointInTime now asAbsoluteSeconds with: PointInTime now asPosixSeconds"

	^ self asPosixSecondsForTimeZone: LocalTimeTransform forSystem! !

!PointInTime methodsFor: 'converting' stamp: 'dtl 10/23/2004 11:23'!
asPosixSecondsForTimeZone: aTimeZone
	"Answer the number of Posix seconds since 01 January, 1970 UTC.
	The count of Posix seconds varies from absolute seconds as leap seconds
	are applied to the calendar."

	^ self absoluteTime - (aTimeZone leapSecondsAt: self)
! !

!PointInTime methodsFor: 'converting' stamp: 'PaulDeBruicker 3/3/2016 15:46'!
asSmalltalkSecondsForTimeZone: aTimeZone
	"Answer the number of seconds since January 1, 1901 in aTimeZone. This
	number is suitable for creating instances of Time and Date."

	| offset leap |
	self absoluteTime isNil
		ifTrue: [ ^ nil ].
	aTimeZone isNil
		ifTrue: [ 
			self notify: 'nil TimeZone'.
			^ nil ].
	offset := aTimeZone localOffsetSecondsAt: self.
	leap := aTimeZone leapSecondsAt: self.
	^ self absoluteTime  + offset - leap+ self class posixOffset! !

!PointInTime methodsFor: 'converting' stamp: ''!
asUtcDateAndTime

	"PointInTime now asUtcDateAndTime"

	^ self class dateAndTimeFromSmalltalkSeconds: self asUtcSmalltalkSeconds! !

!PointInTime methodsFor: 'converting' stamp: ''!
asUtcSmalltalkSeconds
	"Answer the number of seconds since January 1, 1901 in aTimeZone with zero
	offset from UTC. This number is suitable for creating instances of Time and
	Date as they might appear in a Smalltalk system which happened to be
	located near longitude 0."

	"Time dateAndTimeFromSeconds: (PointInTime now asUtcSmalltalkSeconds)"

	self absoluteTime isNil ifTrue: [^ nil].
	^ (self absoluteTime asInteger) + self class posixOffset! !

!PointInTime methodsFor: 'accessing' stamp: 'dtl 11/17/2004 20:03'!
currentLeapSeconds
	"Current number of leap seconds since the Posix epoch. This is independent
	of geographical time zone, but will be reported as zero if the local time
	transform does not recognize leap seconds."

	"PointInTime now currentLeapSeconds"

	^ LocalTimeTransform forSystem leapSecondsAt: self
! !

!PointInTime methodsFor: 'accessing' stamp: 'dtl 11/17/2004 20:04'!
currentOffsetSeconds
	"Current seconds offset from UTC in the local time zone. For a given
	time zone, the value will change at daylight savings time transitions."

	"PointInTime now currentOffsetSeconds"

	^ LocalTimeTransform forSystem localOffsetSecondsAt: self
! !

!PointInTime methodsFor: 'comparing' stamp: 'dtl 10/23/2004 13:17'!
hash

	^ self absoluteTime hash! !

!PointInTime methodsFor: 'printing' stamp: 'PaulDeBruicker 2/14/2016 14:05'!
printOn: aStream
	super printOn: aStream.
	aStream nextPutAll: ' at '.
	absoluteTime printOn: aStream.
	aStream nextPutAll: ' seconds '.
	self asDateAndTime printOn: aStream! !

PointInTime subclass: #PointInTimeNow
	instanceVariableNames: 'updateProcess'
	classVariableNames: 'ThisInstant UpdatePeriod'
	poolDictionaries: ''
	category: 'Time-Examples'!

!PointInTimeNow commentStamp: '<historical>' prior: 0!
I represent the current point in time, as reported by the underlying hardware
or operating system. There is only one instance of me in the system.

My update mechanism is a simple polling loop which updates my view of the
system time periodically and notifies any dependents of the change.!

!PointInTimeNow class methodsFor: 'initialize-release' stamp: 'dtl 1/7/2000 19:47'!
initialize

	"PointInTimeNow initialize"

	self updatePeriod: 200.	"milliseconds"
	ThisInstant isNil ifFalse: [
		ThisInstant release.
		ThisInstant terminateProcess.
		ThisInstant := nil]! !

!PointInTimeNow class methodsFor: 'instance creation' stamp: 'dtl 12/28/1999 20:01'!
new

	self notify: 'use PointInTimeNow>>thisInstant to access my single instance'! !

!PointInTimeNow class methodsFor: 'instance creation' stamp: 'dtl 1/5/2000 20:07'!
thisInstant

	"PointInTimeNow thisInstant"

	ThisInstant isNil ifTrue: [ThisInstant := super new initialize].
	^ ThisInstant! !

!PointInTimeNow class methodsFor: 'initialize-release' stamp: 'dtl 1/7/2000 19:46'!
updatePeriod: milliseconds

	UpdatePeriod := milliseconds! !

!PointInTimeNow methodsFor: 'converting' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
asPointInTime
	"Answer a snapshot of myself at the current time."

	"PointInTimeNow thisInstant asPointInTime"

	| now |
	now := self absoluteTime.
	^ PointInTime new absoluteTime: now! !

!PointInTimeNow methodsFor: 'initialize-release' stamp: 'dtl 1/9/2000 12:35'!
initialize

	"PointInTime initialize"

	self addDependent: self.
	self updateFromSystemClock! !

!PointInTimeNow methodsFor: 'initialize-release' stamp: 'dtl 1/9/2000 13:52'!
release

	"PointInTime release"

	self dependents release.
	^ super release

! !

!PointInTimeNow methodsFor: 'updating' stamp: 'dtl 11/21/2004 11:07'!
setAbsoluteTime

	"PointInTime new setAbsoluteTime asLocalDateAndTime"

	self absoluteTime: (self class absoluteSecondsNow)
! !

!PointInTimeNow methodsFor: 'system clock' stamp: 'dtl 12/30/1999 19:47'!
terminateProcess

	updateProcess isNil ifFalse: [updateProcess terminate].
! !

!PointInTimeNow methodsFor: 'updating' stamp: 'dtl 12/28/1999 20:01'!
update: aParameter

	self setAbsoluteTime
! !

!PointInTimeNow methodsFor: 'system clock' stamp: 'dtl 1/7/2000 19:47'!
updateFromSystemClock
	"Start a polling loop to periodically update my view of the system time."

	self terminateProcess.
	updateProcess := [[true] whileTrue: [
		(Delay forMilliseconds: UpdatePeriod) wait.
		self changed: self]] forkAt: Processor userInterruptPriority
	! !

!TimeZoneDatabase class methodsFor: '*Time-UTC-examples' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
convertDate: aDate time: aTime fromTimeZone: aTimeZone toTimeZone: anotherTimeZone
	"For aDate at aTime in aTimeZone, answer the Date and Time in anotherTimeZone."

	"TimeZoneDatabase convertDate: (TimeZoneDatabase dateFromString: '01.01.2000')
						time: (Time fromSeconds: 0)
						fromTimeZone: (LocalTimeTransform for: 'Europe/Paris')
						toTimeZone: (LocalTimeTransform for: 'America/Detroit')"

	"TimeZoneDatabase convertDate: (TimeZoneDatabase dateFromString: '01.01.2000')
						time: (Time fromSeconds: 0)
						fromTimeZone: (LocalTimeTransform for: 'America/Detroit')
						toTimeZone: (LocalTimeTransform for: 'Australia/Perth')"

	| pointInTime |
	pointInTime := PointInTime date: aDate time: aTime timeZone: aTimeZone.
	pointInTime isNil
		ifTrue: [^ nil]
		ifFalse: [^ pointInTime asDateAndTimeForTimeZone: anotherTimeZone]

! !

!TimeZoneDatabase class methodsFor: '*Time-UTC-examples' stamp: 'dtl 1/1/2000 11:47'!
sortedArrayOfWorldTimesAtPointInTime: aPointInTime
	"For the indicated time, answer a sorted array with the values of this time in each
	local time zone."

	"TimeZoneDatabase sortedArrayOfWorldTimesAtPointInTime: PointInTime now"

	"TimeZoneDatabase sortedArrayOfWorldTimesAtPointInTime: (PointInTime fromSmalltalkSeconds: 0)"

	"TimeZoneDatabase sortedArrayOfWorldTimesAtPointInTime:
		(PointInTime fromSmalltalkSeconds: ((TimeZoneDatabase dateFromString: '01.01.2000') asSeconds)
		inTimeZone: LocalTimeTransform here)"

	| db zones sc idx seconds result |
	db := TimeZoneDatabase systemDatabase.
	db isNil ifTrue: [
		self notify: 'no system time zone database'.
		^ nil].
	zones := db allTimeZones.
	sc := SortedCollection sortBlock: [:x :y | (x at: 3) <= (y at: 3)].
	idx := 0.
	self showProgressFrom: 1 to: zones size withMessage: 'checking times'
		forBlock: [ :bar |
			zones do: [:e |
			idx := idx + 1.
			bar value: idx.
			seconds := aPointInTime asSmalltalkSecondsForTimeZone: e.
			sc add: (Array with: e timeZoneName
								with: (aPointInTime asDateAndTimeForTimeZone: e)
								with: seconds)]].
	result := OrderedCollection new.
	sc do: [:e | result add: (Array with: (e at: 1) with: (e at: 2))].
	^ result asArray
! !

!TimeZoneDatabase class methodsFor: '*Time-UTC-examples' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
worldTimes
	"For the current time, answer a dictionary with the values of this time in each
	local time zone."

	"TimeZoneDatabase worldTimes"

	^ TimeZoneDatabase worldTimesAtPointInTime: PointInTime now

! !

!TimeZoneDatabase class methodsFor: '*Time-UTC-examples' stamp: 'dtl 12/31/1999 16:22'!
worldTimesAtPointInTime: aPointInTime
	"For the indicated time, answer a dictionary with the values of this time in each
	local time zone."

	"TimeZoneDatabase worldTimesAtPointInTime: (PointInTime fromSmalltalkSeconds: 0)"

	| db zones d idx |
	db := TimeZoneDatabase systemDatabase.
	db isNil ifTrue: [
		self notify: 'no system time zone database'.
		^ nil].
	zones := db allTimeZones.
	d := Dictionary new.
	idx := 0.
	self showProgressFrom: 1 to: zones size withMessage: 'checking times'
		forBlock: [ :bar |
			zones do: [:e |
			idx := idx + 1.
			bar value: idx.
			d at: e timeZoneName put: (aPointInTime asDateAndTimeForTimeZone: e)]].
	^ d

! !

!TimeZoneDatabase class methodsFor: '*Time-UTC-examples' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
worldTimesForDate: aDate time: aTime inTimeZone: aTimeZone

	"For aDate at aTime in aTimeZone, answer a dictionary with the values of this time in
	each local time zone in the system time zone database."

	"TimeZoneDatabase worldTimesForDate: (TimeZoneDatabase dateFromString: '01.01.2000')
						time: (Time fromSeconds: 0)
						inTimeZone: LocalTimeTransform here"

	"TimeZoneDatabase worldTimesForDate: (TimeZoneDatabase dateFromString: '01.01.2000')
						time: (Time fromSeconds: 0)
						inTimeZone: (LocalTimeTransform for: 'Australia/Perth')"

	| pointInTime |
	pointInTime := PointInTime date: aDate time: aTime timeZone: aTimeZone.
	pointInTime isNil
		ifTrue: [^ nil]
		ifFalse: [^ TimeZoneDatabase worldTimesAtPointInTime: pointInTime]

! !

!TimeZone class methodsFor: '*time-TimeZones' stamp: 'dtl 8/21/2007 23:14'!
setLocal: tzName
	"Set the local time zone to a time zone identified by tzName. To identify
	suitable time zone names for e.g. Detriot, search the database using:
		TimeZoneDatabase systemDatabase grepFor: 'Detroit'"

	"TimeZone setLocal: 'CET'"
	"TimeZone setLocal: 'EST'"
	"TimeZone setLocal: 'America/Detroit'"
	"TimeZone setLocal: 'no such name'"

	(TimeZoneDatabase systemDatabase timeZoneFor: tzName)
		ifNil: [self notify: '''', tzName asString,
				''' not found in TimeZoneDatabase class>>systemDatabase']
		ifNotNil: [TimeZoneProxy setDynamicTransform: tzName].
	^ TimeZoneDatabase systemDatabase timeZoneFor: tzName
	
! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/13/2004 12:18'!
dstTransitionTimes

	^ #()! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/8/2004 21:23'!
leapSecondsAt: aPointInTime
	"Ignore leap seconds"

	^ 0
! !

!TimeZone methodsFor: '*time-TimeZones' stamp: 'dtl 11/12/2004 10:31'!
localOffsetSecondsAt: aPointInTime

	^ self offset asSeconds! !

TestCase subclass: #DateTimeTests
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones-tests'!

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:21'!
assertEqualExcludingOffset: dtNow and: datNow
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ].
	self assert: datNow offset ~= dtNow offset! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:21'!
assertEqualIncludingOffset: dtNow and: datNow
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond #offset)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ]! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:23'!
testFromSeconds
	| dat dt |
	dat := DateAndTime fromSeconds: 0.
	dt := DateTime fromSeconds: 0.
	self assertEqualIncludingOffset: dt and: dat.
	dat := DateAndTime fromSeconds: 0 offset: 0 seconds.
	dt := DateTime fromSeconds: 0 inTimeZoneNamed: #'America/Detroit'.
	self assertEqualIncludingOffset: dt and: dat! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:13'!
testNow
	| datNow dtNow |
	datNow := DateAndTime now.
	dtNow := DateTime now.
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond #offset)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ]! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:23'!
testOffsetGivesError
	| dtNow |
	dtNow := DateTime now.
	self should: [ dtNow offset: 3600 seconds ] raise: ShouldNotImplement.
	self should: [ DateTime fromSeconds: 0 offset: 0 seconds ] raise: [ ShouldNotImplement ]! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 14:09'!
testPosixOffset
	self assert: DateTime posixOffset equals: 2177452800! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 16:21'!
testYearMonthDay
	| datNow dtNow |
	datNow := DateAndTime year: 2016 month: 3 day: 12.
	dtNow := DateTime year: 2016 month: 3 day: 12.
	self assertEqualIncludingOffset: dtNow and: datNow.
	datNow := DateAndTime
		year: 2016
		month: 3
		day: 12
		offset: 0 seconds.
	dtNow := DateTime year: 2016 month: 3 day: 12.
	self assertEqualExcludingOffset: dtNow and: datNow! !

!DateTimeTests class methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 6/4/2016 13:05'!
testYearMonthDayHourMinuteSecond
	| dat dt |
	dat := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		offset: -18000 seconds.
	dt := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		timeZone: #'America/Detroit'.
	self assertEqualIncludingOffset: dt and: dat.
	dat := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		offset: 0 seconds.
	dt := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		timeZone: #'America/Detroit'.
	self assertEqualExcludingOffset: dt and: dat! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:23'!
assertEqualExcludingOffset: dtNow and: datNow
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ].
	self assert: datNow offset ~= dtNow offset! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:23'!
assertEqualIncludingOffset: dtNow and: datNow
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond #offset)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ]! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:23'!
testFromSeconds
	| dat dt |
	dat := DateAndTime fromSeconds: 0.
	dt := DateTime fromSeconds: 0.
	self assertEqualIncludingOffset: dt and: dat.
	dat := DateAndTime fromSeconds: 0 offset: 0 seconds.
	dt := DateTime fromSeconds: 0 inTimeZoneNamed: #'America/Detroit'.
	self assertEqualIncludingOffset: dt and: dat! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:23'!
testNow
	| datNow dtNow |
	datNow := DateAndTime now.
	dtNow := DateTime now.
	#(#year #month #day #dayOfMonth #dayOfYear #hour #minute #second #nanoSecond #offset)
		do: [ :eachMethod | self assert: (datNow perform: eachMethod) equals: (dtNow perform: eachMethod) ]! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:23'!
testPosixOffset
	self assert: DateTime posixOffset equals: 2177452800! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'pad 3/14/2016 11:30'!
testYearMonthDay
	| datNow dtNow |
	datNow := DateAndTime year: 2016 month: 3 day: 12.
	dtNow := DateTime year: 2016 month: 3 day: 12.
	self assertEqualIncludingOffset: dtNow and: datNow.
	datNow := DateAndTime
		year: 2016
		month: 3
		day: 12
		offset: 3600 seconds.
	dtNow := DateTime year: 2016 month: 3 day: 12.
	self assertEqualExcludingOffset: dtNow and: datNow! !

!DateTimeTests methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 6/4/2016 13:05'!
testYearMonthDayHourMinuteSecond
	| dat dt |
	dat := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		offset: -18000 seconds.
	dt := DateTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		timeZone: #'America/Detroit'.
	self assertEqualIncludingOffset: dt and: dat.
	dat := DateAndTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		offset: 0 seconds.
	dt := DateTime
		year: 2016
		month: 3
		day: 12
		hour: 12
		minute: 15
		second: 16
		timeZone: #'America/Detroit'.
	self assertEqualExcludingOffset: dt and: dat! !

TestCase subclass: #TimeTransformTest
	instanceVariableNames: 'saveLocalTimeZone saveLocalTransform saveLastTransitionTime dstTransitionAbsoluteSeconds dstTransitionPosixSeconds leapSeconds'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones-tests'!

!TimeTransformTest commentStamp: 'dtl 11/25/2012 17:41' prior: 0!
Unit tests for local time transforms (time zone rules), and for integration of time zone rules with date and time classes. Two local time transforms are used, one that makes use of the leap second table (right/America/Detroit), and another that ignores the leap second table (America/Detroit). The tests check for correct time zone offsets before and after the Fall 2004 Daylight Savings transition for Detroit, Michigan USA, and for times before and after a leap second transition. A system time zone database with the necessary time zones must exist prior to running the tests.

See method category "documentation" for an explanation of time representations.

Some of these tests will fail if the TimePlugin is being used. TimePlugin reports the operating system view of Posix seconds, while TimeZoneDatabase calculates Posix seconds from local seconds using the time zone offset. The tests that are expected to file have method comments to that effect. With TimePlugin disabled, all tests should pass (but PointInTime will not have access to the millisecond clock provided by TimePlugin).

!

!TimeTransformTest methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
absoluteDstTransition
	"The first UTC second for the next daylight savings transition in absolute seconds.
	Search the actual time zone table, just in case someone changes the table retroactively."

	"self new absoluteDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs := 1096831000. "A time during October 2004"
	tz := LocalTimeTransform here.
	summerOffset := tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: secs).
	dayAfterTransition := (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	hourAfterTransition := (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	minuteAfterTransition := (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	secondAfterTransition := (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromAbsoluteSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'running' stamp: 'pad 2/16/2016 11:04'!
dstTransitionAbsoluteSeconds
	"macs don't have posix time, only 'right' time"

	self osUnderstandsLeapSeconds
		ifTrue: [ ^ 1099202422 ]
		ifFalse: [ ^ 1099202400 ]! !

!TimeTransformTest methodsFor: 'documentation' stamp: 'dtl 6/14/2009 19:20'!
hasBrokenDateAndTimeConversion
	"The constructors for DateAndTime and other Squeak Chronology classes
	do not handle time zone offsets correctly. They use whatever offset is currently
	in effect, which may or may not be the same as the offset in effect as of
	the date and time for the DateAndTime instance to be created."

	self flag: #FIXME.
	^ true! !

!TimeTransformTest methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
localSmalltalkDstTransition
	"The first local Smalltalk second for the next daylight savings transition in
	the local time zone."

	"self new localSmalltalkDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs := 1096831000 + self posixOffset. "A time during October 2004"
	tz := LocalTimeTransform here.
	summerOffset := tz localOffsetSecondsAt: (PointInTime fromSeconds: secs).
	dayAfterTransition := (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	hourAfterTransition := (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	minuteAfterTransition := (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	secondAfterTransition := (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 17:17'!
osIsUnix
	TimeZoneDatabase platformIsSqueakOrPharo
		ifTrue: [ 
			TimeZoneDatabase platformIsSqueak
				ifTrue: [ ^ Smalltalk platformName ~= 'Mac OS' ]
				ifFalse: [ ^ (Smalltalk at: #OSPlatform) currentPlatformName ~= 'Mac OS' ] ].
	^ true! !

!TimeTransformTest methodsFor: 'running' stamp: 'pad 2/16/2016 11:08'!
osUnderstandsLeapSeconds
	^self osIsUnix! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'pad 2/28/2016 23:32'!
posixDetroitTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'posix/America/Detroit' ]
		ifFalse: [ ^ 'America/Detroit' ]! !

!TimeTransformTest methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
posixDstTransition
	"The first UTC Posix second for the next daylight savings transition. Search the
	actual time zone table, just in case someone changes the table retroactively."

	"self new posixDstTransition"

	| secs tz summerOffset dayAfterTransition hourAfterTransition secondAfterTransition minuteAfterTransition |
	secs := 1096831000. "A time during October 2004"
	tz := LocalTimeTransform here.
	summerOffset := tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: secs).
	dayAfterTransition := (secs to: secs + 3600 * 24 * 100 by: 3600 * 24)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	hourAfterTransition := (dayAfterTransition - (3600 * 24) to: dayAfterTransition by: 3600)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	minuteAfterTransition := (hourAfterTransition - 3600 to: hourAfterTransition by: 60)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	secondAfterTransition := (minuteAfterTransition - 60 to: minuteAfterTransition by: 1)
		detect: [:s | ((tz localOffsetSecondsAt: (PointInTime fromPosixSeconds: s))
				== summerOffset) not].
	^ secondAfterTransition
! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 10/23/2004 12:59'!
posixOffset
	"Difference in seconds between Smalltalk epoch and Posix epoch."
	"52 * 365 + (17 * 366) * 24 * 60 * 60"

	^ 2177452800! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 12:22'!
posixPhoenixTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'posix/America/Phoenix' ]
		ifFalse: [ ^ 'America/Phoenix' ]! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 12:22'!
posixUTCTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'posix/UTC' ]
		ifFalse: [ ^ 'UTC' ]! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/14/2004 17:22'!
primPosixTimeAndLocalOffset
	"Array of two integers, with number of seconds since since Jan 1, 1970 UTC,
	and the local offset for the time zone in effect for this system."

	<primitive: 'primitivePosixTimeAndLocalOffset' module: 'TimeFunctions'>
	^ nil! !

!TimeTransformTest methodsFor: 'private' stamp: 'dtl 11/14/2004 17:15'!
primSecondsClock
	"Answer the number of seconds since 00:00 on the morning of
	 January 1, 1901 (a 32-bit unsigned number).
	 Essential. See Object documentation whatIsAPrimitive. "

	<primitive: 137>
	self primitiveFailed! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 17:13'!
rightChicagoTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'right/America/Chicago' ]
		ifFalse: [ ^ 'America/Chicago' ]! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'pad 2/28/2016 23:32'!
rightDetroitTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'right/America/Detroit' ]
		ifFalse: [ ^ 'America/Detroit' ]! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 17:14'!
rightLosAngelesTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'right/America/Los_Angeles' ]
		ifFalse: [ ^ 'America/Los_Angeles' ]! !

!TimeTransformTest methodsFor: 'time zone names' stamp: 'PaulDeBruicker 2/13/2016 12:22'!
rightUTCTimeZoneName
	self osIsUnix
		ifTrue: [ ^ 'right/UTC' ]
		ifFalse: [ ^ 'UTC' ]! !

!TimeTransformTest methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
setDstTransitionWatcherFall2004

	"Set the DST transition watcher to think that the last DST transition
	happened in the Fall of 2004. This will permit tests that rely on the DST
	transition watcher to work properly in the future. This method must be
	called after the time zone setting is changed, because the DST transition
	watcher is automatically reset whenever such a change occurs."

	| lateFall2004 newLastTransitionTick |
	lateFall2004 := PointInTime fromAbsoluteSeconds: 1100000000.
	newLastTransitionTick := TimeZoneDstTransitionWatcher
		clockWatcher probableLastTransitionTickBefore: lateFall2004.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: newLastTransitionTick.
! !

!TimeTransformTest methodsFor: 'private' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
setDstTransitionWatcherSpring2004

	"Set the DST transition watcher to think that the last DST transition
	happened in the Spring of 2004. This will permit tests that rely on the DST
	transition watcher to work properly in the future. This method must be
	called after the time zone setting is changed, because the DST transition
	watcher is automatically reset whenever such a change occurs."

	| earlySummer2004 newLastTransitionTick |
	earlySummer2004 := PointInTime fromAbsoluteSeconds: 1084440000.
	newLastTransitionTick := TimeZoneDstTransitionWatcher
		clockWatcher probableLastTransitionTickBefore: earlySummer2004.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: newLastTransitionTick.
! !

!TimeTransformTest methodsFor: 'running' stamp: 'PaulDeBruicker 2/14/2016 14:51'!
setUp
	"Save the system time zone setting"

	saveLocalTransform := TimeZoneDatabase systemDatabase defaultTimeZone.
	saveLocalTimeZone := DateAndTime localTimeZone.
	saveLastTransitionTime := TimeZoneDstTransitionWatcher clockWatcher transitionTick.	"Configure for time zone proxy to the database"
	TimeZoneProxy setDynamicTransform.	"Fall 2004 DST transition for right/America/Detroit, in atomic clock seconds"
	dstTransitionAbsoluteSeconds := self dstTransitionAbsoluteSeconds.	"Fall 2004 DST transition for posix/America/Detroit, in Posix seconds, ignoring leap seconds"
	dstTransitionPosixSeconds := 1099202400.	"Number of leap seconds as of the transition time"
	leapSeconds := 22! !

!TimeTransformTest methodsFor: 'documentation' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
squeakSecondsClockInfo

	"(StringHolder new acceptContents: self new squeakSecondsClockInfo; yourself)
		openLabel: 'Squeak seconds clock' "

	^ PointInTime squeakSecondsClockInfo
! !

!TimeTransformTest methodsFor: 'running' stamp: 'dtl 11/14/2004 16:21'!
tearDown
	"Restore the default time zone"

	TimeZoneDatabase systemDatabase defaultLocation: saveLocalTransform timeZoneName.
	DateAndTime localTimeZone: saveLocalTimeZone.
	TimeZoneDstTransitionWatcher clockWatcher transitionTick: saveLastTransitionTime

! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAbsoluteTimeToDateAndTime
	"Check conversion from absolute time to DateAndTime local seconds as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToDateAndTime) debug"

	| interval expectedStrings dateAndTimeStrings |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	interval := dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	expectedStrings := #(
		'2004-10-31T01:59:58-04:00'
		'2004-10-31T01:59:59-04:00'
		'2004-10-31T01:00:00-05:00'
		'2004-10-31T01:00:01-05:00').
	dateAndTimeStrings := interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asDateAndTime printString].
	self assert: dateAndTimeStrings = expectedStrings! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAbsoluteTimeToDateAndTimeSeconds
	"Check conversion from absolute time to DateAndTime local seconds as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToDateAndTimeSeconds) debug"

	| interval dateAndTimes dateAndTimeSeconds validCombinationsOfExpectedSecondsValues |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	interval := dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	validCombinationsOfExpectedSecondsValues := Array
		with: #(3276640798 3276640799 3276637200 3276637201) "current DST offset 4 hours, contiguous local seconds"
		with: #(3276640798 3276640799 3276640800 3276640801) "current DST offset 4 hours, skipped an hour"
		with: #(3276637198 3276637199 3276637200 3276637201) "current DST offset 5 hours, contiguous local seconds"
		with: #(3276637198 3276637199 3276633600 3276633601). "current DST offset 5 hours, skipped an hour"
	dateAndTimes := interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asDateAndTime].
	"This conversion is ambiguous for the hour following DST transition"
	dateAndTimeSeconds := dateAndTimes collect: [:dt | dt localSecondsSinceEpoch].
	"The third and fourth elements of the array are ambiguous, may be off by 1 hour"
	self assert: (validCombinationsOfExpectedSecondsValues includes: dateAndTimeSeconds)
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAbsoluteTimeToLocalSmalltalkSeconds
	"Check conversion from absolute time to local seconds count as we cross 
	a DST transition."

	"(self new setTestSelector: #testAbsoluteTimeToLocalSmalltalkSeconds) debug"

	| interval localSmalltalkSeconds expectedLocalSmalltalkSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	interval := dstTransitionAbsoluteSeconds - 2 to: dstTransitionAbsoluteSeconds + 1.
	localSmalltalkSeconds := interval
				collect: [:abs | (PointInTime fromAbsoluteSeconds: abs) asLocalSmalltalkSeconds].
	expectedLocalSmalltalkSeconds := #(3276640798 3276640799 3276637200 3276637201).
	self assert: localSmalltalkSeconds = expectedLocalSmalltalkSeconds.
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testAddDurationAcrossDstTransition

	"(self new setTestSelector: #testAddDurationAcrossDstTransition) debug"

	| midnight fourHoursLater duration |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	midnight := '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater := '2004-10-31T03:00:00-05:00' asDateAndTime.
	duration := Duration seconds: (4 * 3600).
	self assert: midnight + duration = fourHoursLater
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAddDurationAcrossLeapSecondTransition

	"(self new setTestSelector: #testAddDurationAcrossLeapSecondTransition) debug"

self osUnderstandsLeapSeconds ifTrue:[	
	| aLeapSecondTransitionTime ptFirst ptTenSecondsLater |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	aLeapSecondTransitionTime := 915148821. "From the leap second table in right/America/Detroit"
	ptFirst := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2.
	ptTenSecondsLater := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2 + 10.
	self assert: (ptTenSecondsLater asAbsoluteSeconds - ptFirst asAbsoluteSeconds) = 10.
	self assert: (ptTenSecondsLater asPosixSeconds - ptFirst asPosixSeconds) = 9.
	self assert: (ptTenSecondsLater asLocalSmalltalkSeconds - ptFirst asLocalSmalltalkSeconds) = 9.
	self assert: (ptFirst asDateAndTime + (Duration seconds: 9)) = ptTenSecondsLater asDateAndTime]! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAmbiguousConversionAfterDst
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	the DST transition."

	"(self new setTestSelector: #testAmbiguousConversionAfterDst) debug"

	| halfHourBefore halfHourAfter smalltalkSecondsBefore smalltalkSecondsAfter |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	halfHourBefore := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - 1800.
	halfHourAfter := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 1800.
	self setDstTransitionWatcherSpring2004.
	smalltalkSecondsBefore := halfHourBefore asLocalSmalltalkSeconds.
	self setDstTransitionWatcherFall2004.
	smalltalkSecondsAfter := halfHourAfter asLocalSmalltalkSeconds.
	self deny: halfHourBefore = halfHourAfter.
	self assert: smalltalkSecondsBefore = smalltalkSecondsAfter
! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAmbiguousConversionForLeapSeconds
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	a leap second transition."

	"(self new setTestSelector: #testAmbiguousConversionForLeapSeconds) debug"
	self osUnderstandsLeapSeconds
		ifTrue: [
	| tz leapSecondTransition oneSecondBeforeTransition ptArray twoSecondsAfterTransition |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	tz := TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName.
	leapSecondTransition := (PointInTime fromAbsoluteSeconds: 915148821) asSmalltalkSecondsForTimeZone: tz.
	
	"One second before a leap second transition, conversion to local seconds is unambiguous."
	oneSecondBeforeTransition := leapSecondTransition - 1.
	ptArray := PointInTime allAbsoluteTimesCorrespondingTo: oneSecondBeforeTransition inTimeZone: tz.
	self assert: ptArray size == 1.
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = oneSecondBeforeTransition.

	"During the two second duration of the leap second transition, conversion to local seconds is ambiguous.
	There are two atomic clock PointInTime seconds corresponding to a single wall clock second."
	ptArray := PointInTime allAbsoluteTimesCorrespondingTo: leapSecondTransition inTimeZone: tz.
	self assert: ptArray size == 2.
	self deny: ptArray first = ptArray second.
	"But both of the PointInTime values translate back to the same local Smalltalk second count."
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = leapSecondTransition.
	self assert: (ptArray second asSmalltalkSecondsForTimeZone: tz) = leapSecondTransition.

	"Two seconds after the leap seconds transition, conversion to local seconds is unambiguous again."
	twoSecondsAfterTransition := leapSecondTransition + 1.
	ptArray := PointInTime allAbsoluteTimesCorrespondingTo: twoSecondsAfterTransition inTimeZone: tz.
	self assert: ptArray size == 1.
	self assert: (ptArray first asSmalltalkSecondsForTimeZone: tz) = twoSecondsAfterTransition.]

! !

!TimeTransformTest methodsFor: 'testing-local conversion' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testAmbiguousConversionToLocalSeconds
	"Demonstrate ambiguity of translation from PointInTime to Smalltalk seconds during
	the DST transition."

	"(self new setTestSelector: #testAmbiguousConversionToLocalSeconds) debug"

	"One second before the fall 2004 DST transition, conversion to local seconds is unambiguous."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276637199
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 1.

	"During the hour after the fall 2004 DST transition, conversion to local seconds is ambiguous."
	"There are two PointInTime values for a Smalltalk second count."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276637200
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 2.
	"But both of the PointInTime values translate back to the same local Smalltalk second count."
	self assert:
		((PointInTime allAbsoluteTimesCorrespondingTo: 3276637200
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit'))
				collect: [:e | e asSmalltalkSecondsForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')])
			asSet size == 1.
	"This continues to be the case for the next hour."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276640799
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 2.
	self assert:
		((PointInTime allAbsoluteTimesCorrespondingTo: 3276640799
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit'))
				collect: [:e | e asSmalltalkSecondsForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')])
			asSet size == 1.

	"One hour after the fall 2004 DST transition, conversion to local seconds is unambiguous again."
	self assert:
		(PointInTime allAbsoluteTimesCorrespondingTo: 3276640800
			inTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit')) size == 1.

! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testAsTime
	"Ensure that a Time created from a DateAndTime is the right magnitude."

	"(self new setTestSelector: #testAsTime) debug"

	| dt t dtSecs tSecs |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	dt := DateAndTime now.
	t := dt asTime.
	dtSecs := (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs := (t hour * 3600) + (t minute * 60) + t second.
	"dtSecs is a whole integer, so values are expected to match to within one second"
	self assert: (dtSecs - tSecs) abs < 1.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	dt := DateAndTime now.
	t := dt asTime.
	dtSecs := (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs := (t hour * 3600) + (t minute * 60) + t second.
	"dtSecs is a whole integer, so values are expected to match to within one second"
	self assert: (dtSecs - tSecs) abs < 1
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testConversionToLocalSmalltalkSeconds
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	The Smalltalk seconds clock reports seconds in wall clock time in the local
	time zone, and is unaware of time zone offsets and leap seconds. The
	PointInTime clock reports actual elapsed Caesium clock seconds since
	the Posix epoch. As leap seconds occur, the two clocks become out of
	sync. If the local time transform is aware of leap seconds, the difference
	between the two clocks will appear in the conversion between PointInTime
	seconds and Smalltalk seconds."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testConversionToLocalSmalltalkSeconds) debug"

	| pt tSec tpSec absoluteSeconds leap posixSeconds offset calculatedSmalltalkSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime now.
	tSec := Time totalLocalSeconds.
	tpSec := pt asLocalSmalltalkSeconds.
	self assert: (tSec - tpSec) abs < 1.
	absoluteSeconds := pt asAbsoluteSeconds.
	leap := LocalTimeTransform here leapSecondsAt: pt.
	posixSeconds := absoluteSeconds + PointInTime posixOffset - leap.
	offset := LocalTimeTransform here localOffsetSecondsAt: pt.
	calculatedSmalltalkSeconds := posixSeconds + offset.
	self assert: (tSec - calculatedSmalltalkSeconds) abs < 1.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	pt := PointInTime now.
	tSec := Time totalLocalSeconds.
	tpSec := pt asLocalSmalltalkSeconds.
	self assert: (tSec - tpSec) abs < 1.
	absoluteSeconds := pt asAbsoluteSeconds.
	leap := LocalTimeTransform here leapSecondsAt: pt.
	posixSeconds := absoluteSeconds + PointInTime posixOffset - leap.
	offset := LocalTimeTransform here localOffsetSecondsAt: pt.
	calculatedSmalltalkSeconds := posixSeconds + offset.
	self assert: (tSec - calculatedSmalltalkSeconds) abs < 1! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'pad 2/16/2016 11:11'!
testConversionToLocalSmalltalkSeconds2
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	The Smalltalk seconds clock reports seconds in wall clock time in the local
	time zone, and is unaware of time zone offsets and leap seconds. The
	PointInTime clock reports actual elapsed Caesium clock seconds since
	the Posix epoch. As leap seconds occur, the two clocks become out of
	sync. If the local time transform is aware of leap seconds, the difference
	between the two clocks will appear in the conversion between PointInTime
	seconds and Smalltalk seconds."
	self osUnderstandsLeapSeconds
		ifTrue: [
	| tpSec leap |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	tpSec := 1098000000. "2004-10-17T04:00:00-04:00"
	leap := LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: leap == 22
]
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testConversionToLocalSmalltalkSecondsNoLeapSeconds
	"Check conversion of PointInTime to local Smalltalk wall clock seconds.
	This test uses a time zone with no leap second table, so the conversion
	is expected to be exact."

	"(self new setTestSelector: #testConversionToLocalSmalltalkSecondsNoLeapSeconds) debug"

	| tpSec tSec |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	tpSec := PointInTime now asLocalSmalltalkSeconds.
	tSec := Time totalLocalSeconds.	"The values may be slightly different, but less than one second"
	self assert: tSec - tpSec < 1.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	tpSec := PointInTime now asLocalSmalltalkSeconds.
	tSec := Time totalLocalSeconds.	"The values may be slightly different, but less than one second"
	self assert: tSec - tpSec < 1! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testConvertLocalSecondsToPointInTime
	"Verify offset between UTC and America/Detroit, and leap second difference
	between America/Detroit and right/America/Detroit."

	self osUnderstandsLeapSeconds
		ifTrue: [	| s pt1 pt2 pt3 |
	s := 3275329734. "2004-10-15T17:48:54-04:00"

	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt1 := PointInTime fromSeconds: s.
	self assert: s = pt1 asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	pt2 := PointInTime fromSeconds: s.
	self assert: s = pt2 asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	pt3 := PointInTime fromSeconds: s.
	self assert: s = pt3 asLocalSmalltalkSeconds.

	self assert: (pt2 absoluteTime - pt3 absoluteTime) = 14400. "time zone offset"
	self assert: (pt1 absoluteTime - pt2 absoluteTime) = 22. "leap seconds"]! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testCreateDateAndTimeArray

	"(self new setTestSelector: #testCreateDateAndTimeArray) debug"

	| date time z1 z2 z3 pt1 pt2 pt3 dt1 dt2 dt3 |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	date := '23 Nov, 2004' asDate.
	time := '12:00' asTime.
	z1 := TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName.
	z2 := TimeZoneDatabase systemDatabase timeZoneFor: self rightChicagoTimeZoneName.
	z3 := TimeZoneDatabase systemDatabase timeZoneFor: self rightLosAngelesTimeZoneName.
	pt1 := PointInTime date: date time: time timeZone: z1.
	pt2 := PointInTime date: date time: time timeZone: z2.
	pt3 := PointInTime date: date time: time timeZone: z3.
	dt1 := pt1 asDateAndTimeForTimeZone: z1.
	dt2 := pt2 asDateAndTimeForTimeZone: z2.
	dt3 := pt3 asDateAndTimeForTimeZone: z3.
	self assert: dt1 = '2004-11-23T12:00:00-05:00' asDateAndTime.
	self assert: dt2 = '2004-11-23T12:00:00-06:00' asDateAndTime.
	self assert: dt3 = '2004-11-23T12:00:00-08:00' asDateAndTime.

	date := '23 Aug, 2004' asDate. "Daylight savings time is in effect"
	time := '12:00' asTime.
	z1 := TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName.
	z2 := TimeZoneDatabase systemDatabase timeZoneFor: self rightChicagoTimeZoneName.
	z3 := TimeZoneDatabase systemDatabase timeZoneFor: self rightLosAngelesTimeZoneName.
	pt1 := PointInTime date: date time: time timeZone: z1.
	pt2 := PointInTime date: date time: time timeZone: z2.
	pt3 := PointInTime date: date time: time timeZone: z3.
	dt1 := pt1 asDateAndTimeForTimeZone: z1.
	dt2 := pt2 asDateAndTimeForTimeZone: z2.
	dt3 := pt3 asDateAndTimeForTimeZone: z3.
	self assert: dt1 = '2004-08-23T12:00:00-04:00' asDateAndTime.
	self assert: dt2 = '2004-08-23T12:00:00-05:00' asDateAndTime.
	self assert: dt3 = '2004-08-23T12:00:00-07:00' asDateAndTime.
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeAtDstTransition
	"Check DateAndTime at the first second of the Fall 2004 DST period in right/America/Detroit.
	The first Smalltalk second count for the transition occurs two hours after midnight, at
	which time the wall clock is set back to 1:00am."

	"(self new setTestSelector: #testDateAndTimeAtDstTransition) debug"

	| dt localSecondCount pt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherFall2004.	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-05:00'.	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	localSecondCount := self localSmalltalkDstTransition.
	dt := DateAndTime fromLocalSeconds: localSecondCount.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: [ 
			"Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
			self assert: localSecondCount = (PointInTime fromSmalltalkSeconds: localSecondCount) asLocalSmalltalkSeconds ]
		ifFalse: [ self assert: dt offset = (Duration seconds: -5 * 60 * 60) ].
	self assert: (PointInTime fromSmalltalkSeconds: localSecondCount) asLocalSmalltalkSeconds = localSecondCount.
	self assert: dt localSecondsSinceEpoch = localSecondCount! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeAtDstTransitionNoLeapSeconds
	"Check DateAndTime at the first second of the Fall 2004 DST period in posix/America/Detroit.
	The first Smalltalk second count for the transition occurs two hours after midnight, at
	which time the wall clock is set back to 1:00am."

	"(self new setTestSelector: #testDateAndTimeAtDstTransitionNoLeapSeconds) debug"

	| dt localSecondCount pt |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	localSecondCount := self localSmalltalkDstTransition.
	self setDstTransitionWatcherFall2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	localSecondCount := self localSmalltalkDstTransition.
	dt := DateAndTime fromLocalSeconds: localSecondCount.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert:  localSecondCount = (PointInTime fromSmalltalkSeconds: localSecondCount) asLocalSmalltalkSeconds]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)].

	self assert: (PointInTime fromSmalltalkSeconds: localSecondCount)
		asLocalSmalltalkSeconds = localSecondCount.
	self assert: dt localSecondsSinceEpoch = localSecondCount
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeConversion
	"Check DateAndTime at midnight before the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeConversion) debug"

	| midnight s dt1 dt2 |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	midnight := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	s := midnight asLocalSmalltalkSeconds.
	dt1 := DateAndTime fromLocalSeconds: s.
	dt2 := midnight asDateAndTime.
	self assert: dt1 seconds = dt2 seconds.
	self assert: dt1 minutes = dt2 minutes.
	self assert: dt1 hours = dt2 hours.
	self assert: dt1 day = dt2 day.
	self assert: dt1 julianDayNumber = dt2 julianDayNumber.
	self assert: dt1 month = dt2 month.
	self assert: dt1 year = dt2 year.
	self assert: dt1 nanoSecond = dt2 nanoSecond.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["offset may be incorrect, DateAndTime does not understand DST transition"]
		ifFalse: [self assert: dt1 offset = dt2 offset.
				self assert: dt1 = dt2]
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/3/2016 13:50'!
testDateAndTimeFromSeconds
	"Test conversion using a time zone with a leap second table."

	| s dt dta |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	s := Time totalLocalSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	dta := Time dateAndTimeFromLocalSeconds: s.
	self assert: dt asDate asString = dta first asString.	"#asString becaues start offsets may differ"
	self assert: dt asTime = dta last! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/3/2016 13:50'!
testDateAndTimeFromSecondsNoLeapSeconds
	"Test conversion using a time zone with no leap second table."

	| s dt dta |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	s := Time totalLocalSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	dta := Time dateAndTimeFromLocalSeconds: s.
	self assert: dt asDate asString = dta first asString.	"#asString becaues start offsets may differ"
	self assert: dt asTime = dta last! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/3/2016 13:50'!
testDateAndTimeFromSecondsUTC
	"Test conversion using a time zone with no offset or leap seconds."

	| s dt dta |
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	s := Time totalLocalSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	dta := Time dateAndTimeFromSeconds: s.
	self assert: dt asDate = dta first.
	self assert: dt asTime = dta last! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeImmediatelyAfter
	"Check DateAndTime for the first second after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyAfter) debug"

	| dt pt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherFall2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition + 1.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:01-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	dt := DateAndTime fromLocalSeconds: self localSmalltalkDstTransition + 1.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)].
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeImmediatelyAfterNoLeapSeconds
	"Check DateAndTime for the first second after the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyAfterNoLeapSeconds) debug"

	| dt pt |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self setDstTransitionWatcherFall2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition + 1.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:01-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	dt := DateAndTime fromLocalSeconds: self localSmalltalkDstTransition + 1.
	self assert: dt seconds == 1.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)].
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeImmediatelyBefore
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyBefore) debug"

	| dt pt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherSpring2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition - 1.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:59:59-04:00'.
	
	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	dt := DateAndTime fromLocalSeconds: self localSmalltalkDstTransition - 1.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -4 * 60 * 60)].
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeImmediatelyBeforeNoLeapSeconds
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeImmediatelyBeforeNoLeapSeconds) debug"

	| dt pt |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	self setDstTransitionWatcherSpring2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromSeconds: self localSmalltalkDstTransition - 1.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:59:59-04:00'.
	
	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	dt := DateAndTime fromLocalSeconds: self localSmalltalkDstTransition - 1.
	self assert: dt seconds == 59.
	self assert: dt minutes == 59.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -4 * 60 * 60)].

! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/3/2016 13:50'!
testDateAndTimeNowSeconds
	"(self new setTestSelector: #testDateAndTimeNowSeconds) debug"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	| clockSeconds dt dtSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	clockSeconds := Time totalLocalSeconds.
	dt := DateAndTime now.
	dtSeconds := dt asSeconds.
	self assert: (clockSeconds - dtSeconds) abs < 1.01.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds := Time totalLocalSeconds.
	dt := DateAndTime now.
	dtSeconds := dt asSeconds.
	self assert: (clockSeconds - dtSeconds) abs < 1.01! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOffsetDuration

	"(self new setTestSelector: #testDateAndTimeOffsetDuration) debug"

	| tt pt local utc offsetSeconds tzOffsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherFall2004.
	tt := self localSmalltalkDstTransition.

	pt := PointInTime fromSeconds: tt.
	local := pt asDateAndTime.
	utc := local asUTC.
	offsetSeconds := local offset asSeconds - utc offset asSeconds.
	tzOffsetSeconds := LocalTimeTransform here
		localOffsetSecondsAt: (PointInTime fromSmalltalkSeconds: tt).
	self assert: offsetSeconds = tzOffsetSeconds.

	local := DateAndTime fromLocalSeconds: tt.
	utc := local asUTC.
	offsetSeconds := local offset asSeconds - utc offset asSeconds.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				offsetSeconds := local offset asSeconds - utc offset asSeconds]
		ifFalse: [self assert: offsetSeconds = tzOffsetSeconds].
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOneDayAfter
	"Check DateAndTime for 24 hours after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneDayAfter) debug"

	| oneDay dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	oneDay := 3600 * 24.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + oneDay).
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 11.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453311.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-11-01T01:00:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 11.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453311.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)].

! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOneDayBefore
	"Check DateAndTime for 24 hours prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneDayBefore) debug"

	| oneDay dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	oneDay := 3600 * 24.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - oneDay).
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453309.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-30T02:00:00-04:00'.
	
	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453309.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -4 * 60 * 60)].

! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOneHalfHourAfter
	"Check DateAndTime for one half hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHalfHourAfter) debug"

	| dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherFall2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 1800.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 30.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:30:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 30.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)]


! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOneHourAfter
	"Check DateAndTime for one hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHourAfter) debug"

	| dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherFall2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 3600.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T02:00:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + 3600.
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 2.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)]

! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeOneHourBefore
	"Check DateAndTime for one hour prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeOneHourBefore) debug"

	| dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherSpring2004.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - 3600.
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T01:00:00-04:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - 3600.
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 1.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -4 * 60 * 60)]

! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeSummer2004EST
	"Check 2004-09-05T12:00:00-04:00 using Posix seconds, absolute seconds, and local Smalltalk seconds."

	"(self new setTestSelector: #testDateAndTimeSummer2004EST) debug"

	| posixSeconds absoluteSeconds ptPosix ptAbsolute offset leap smalltalkSeconds ptSmalltalk |

	"Test with a time transform that includes leap seconds"
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	offset := -4 * 3600.
	leap :=self osUnderstandsLeapSeconds ifTrue:[22] ifFalse:[0].

	"2004-09-05T16:00:00+00:00 Posix time, ignoring leap seconds"
	posixSeconds := 1094400000.
	"Add leap seconds for the number of elapsed atomic clock seconds"
	absoluteSeconds := posixSeconds + leap.
	"Adjust by difference in Posix epoch versus Smalltalk epoch, plus time zone offset"
	smalltalkSeconds := posixSeconds + self posixOffset + offset.

	ptPosix := PointInTime fromPosixSeconds: posixSeconds.
	ptAbsolute := PointInTime fromAbsoluteSeconds: absoluteSeconds.
	ptSmalltalk := PointInTime fromSmalltalkSeconds: smalltalkSeconds.

	"The absolute times should all be the same"
	self assert: ptPosix absoluteTime = ptAbsolute absoluteTime.
	self assert: ptSmalltalk absoluteTime = ptAbsolute absoluteTime.

	"Check commutative conversions"
	self assert: ptAbsolute = (PointInTime fromAbsoluteSeconds: ptAbsolute asAbsoluteSeconds).
	self assert: ptAbsolute = (PointInTime fromPosixSeconds: ptAbsolute asPosixSeconds).
	self assert: ptAbsolute = (PointInTime fromSmalltalkSeconds: ptAbsolute asLocalSmalltalkSeconds).

	"Run the same tests for a time transform that ignores leap seconds"
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	offset := -4 * 3600.
	leap := 0.

	"2004-09-05T16:00:00+00:00 Posix time, ignoring leap seconds"
	posixSeconds := 1094400000.
	"Add leap seconds for the number of elapsed atomic clock seconds"
	absoluteSeconds := posixSeconds + leap.
	"Adjust by difference in Posix epoch versus Smalltalk epoch, plus time zone offset"
	smalltalkSeconds := posixSeconds + self posixOffset + offset.

	ptPosix := PointInTime fromPosixSeconds: posixSeconds.
	ptAbsolute := PointInTime fromAbsoluteSeconds: absoluteSeconds.
	ptSmalltalk := PointInTime fromSmalltalkSeconds: smalltalkSeconds.

	"The absolute times should all be the same"
	self assert: ptPosix absoluteTime = ptAbsolute absoluteTime.
	self assert: ptSmalltalk absoluteTime = ptAbsolute absoluteTime.

	"Check commutative conversions"
	self assert: ptAbsolute = (PointInTime fromAbsoluteSeconds: ptAbsolute asAbsoluteSeconds).
	self assert: ptAbsolute = (PointInTime fromPosixSeconds: ptAbsolute asPosixSeconds).
	self assert: ptAbsolute = (PointInTime fromSmalltalkSeconds: ptAbsolute asLocalSmalltalkSeconds).
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeThreeHoursAfter
	"Check DateAndTime at three hours after the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeThreeHoursAfter) debug"

	| dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 3).
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 4.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T04:00:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 3).
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 4.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)]
! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeTwoHoursAfter
	"Check DateAndTime at two hours after the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeTwoHoursAfter) debug"

	| dt pt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 2).
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 3.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -5 * 60 * 60).
	self assert: dt printString = '2004-10-31T03:00:00-05:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 2).
	s := pt asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 3.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -5 * 60 * 60)]


! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDateAndTimeTwoHoursBefore
	"Check DateAndTime at midnight before the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testDateAndTimeTwoHoursBefore) debug"

	| dt midnight s pt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.

	"Convert directly from a PointInTime. Expect offset to be correct."
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	self assert: pt = pt asDateAndTime asPointInTime.
	dt := pt asDateAndTime.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 0.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self assert: dt offset = (Duration seconds: -4 * 60 * 60).
	self assert: dt printString = '2004-10-31T00:00:00-04:00'.

	"Instantiate indirectly using local Smalltalk seconds. Offset may be incorrect."
	midnight := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	s := midnight asLocalSmalltalkSeconds.
	dt := DateAndTime fromLocalSeconds: s.
	self assert: dt seconds == 0.
	self assert: dt minutes == 0.
	self assert: dt hours == 0.
	self assert: dt month == 10.
	self assert: dt year == 2004.
	self assert: dt julianDayNumber = 2453310.
	self assert: dt nanoSecond == 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: dt offset = DateAndTime now offset]
		ifFalse: [self assert: dt offset = (Duration seconds: -4 * 60 * 60)]

! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 3/3/2016 15:38'!
testDateAtDstTransition
	"Check Date at a DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateAtDstTransition) debug"

	| dt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	dt := Date fromLocalSeconds: self localSmalltalkDstTransition.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 3/4/2016 14:17'!
testDateAtDstTransitionNoLeapSeconds
	"Check Date at a DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateAtDstTransitionNoLeapSeconds) debug"

	| dt |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	dt := Date fromLocalSeconds: self localSmalltalkDstTransition.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 3/4/2016 14:17'!
testDateFirstSecondOfDayOfDstTransition
	"Check Date at the first second of the day on which a DST transition occurs
	in right/America/Detroit."

	"(self new setTestSelector: #testDateFirstSecondOfDayOfDstTransition) debug"

	| dt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self setDstTransitionWatcherSpring2004.
	s := self localSmalltalkDstTransition - (2 * 3600).
	dt := Date fromLocalSeconds: s.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 3/3/2016 23:27'!
testDateFirstSecondOfDayOfDstTransitionNoLeapSeconds
	"Check Date at the first second of the day on which a DST transition occurs
	in posix/America/Detroit."

	"(self new setTestSelector: #testDateFirstSecondOfDayOfDstTransitionNoLeapSeconds) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	self setDstTransitionWatcherSpring2004.
	s := self localSmalltalkDstTransition - (2 * 3600).
	dt := Date fromLocalSeconds: s.
	self assert: dt yyyymmdd = '2004-10-31'! !

!TimeTransformTest methodsFor: 'testing-DateAndTime' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testDateFromYearMonthDay
	"Check DateAndTime creation in different DST periods."

	"(self new setTestSelector: #testDateFromYearMonthDay) debug"

	| dt1 dt2 |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	dt1 := DateAndTime year: 2004 month: 'Nov' day: 9.
	self assert: dt1 year = 2004.
	self assert: dt1 month = 11.
	self assert: dt1 dayOfMonth = 9.
	self assert: dt1 hour = 0.
	self assert: dt1 minute = 0.
	self assert: dt1 second = 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Squeak DateAndTime just uses current offset (this is incorrect)"
			self assert: dt1 offset asSeconds = DateAndTime now offset asSeconds]
		ifFalse: ["This is how it should work"
			self assert: dt1 offset asSeconds = (3600 * -5)].

	dt2 := DateAndTime year: 2004 month: 'Aug' day: 9.
	self assert: dt2 year = 2004.
	self assert: dt2 month = 8.
	self assert: dt2 dayOfMonth = 9.
	self assert: dt2 hour = 0.
	self assert: dt2 minute = 0.
	self assert: dt2 second = 0.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Squeak DateAndTime just uses current offset (this is incorrect)"
			self assert: dt2 offset asSeconds = DateAndTime now offset asSeconds]
		ifFalse: ["This is how it should work"
			self assert: dt2 offset asSeconds = (3600 * -4)]
! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 2/13/2016 17:19'!
testDateInDstPeriods
	"Check the format of a Date posix/America/Detroit, before and after DST transition.
	The start times should show different UTC offsets before and after a DST
	transition. This is required for correct calculation of Date>>asSeconds."

	"(self new setTestSelector: #testDateInDstPeriods) debug"

	| d secondCount1 secondCount2 pt1 pt2 hourOffsetSmalltalkSeconds hourOffsetAbsoluteSeconds |
	self hasBrokenDateAndTimeConversion
		ifTrue: [self flag: #FIXME. "Date and DateAndTime broken for DST conversions"
				^ self "bypass this test entirely until Date can be fixed"].
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	d := Date fromString: '23 Nov, 2004'.
	self assert: d start asString = '2004-11-23T00:00:00-05:00'.
	self assert: d duration asSeconds = (3600 * 24).
	pt1 := d start asPointInTime.
	secondCount1 := d asSeconds.

	d := Date fromString: '23 Aug, 2004'.
	self assert: d start asString = '2004-08-23T00:00:00-04:00'.
	self assert: d duration asSeconds = (3600 * 24).
	pt2 := d start asPointInTime.
	secondCount2 := d asSeconds.
	hourOffsetSmalltalkSeconds := (secondCount2 - secondCount1) \\ (24 * 3600).
	self assert: hourOffsetSmalltalkSeconds = 0.
	hourOffsetAbsoluteSeconds := (pt1 asAbsoluteSeconds - pt2 asAbsoluteSeconds) \\ (24 * 3600).
	self assert: hourOffsetAbsoluteSeconds = 3600.

	d := Date fromString: '23 Nov, 2003'.
	self assert: d start asString = '2003-11-23T00:00:00-05:00'.
	self assert: d duration asSeconds = (3600 * 24).

	d := Date fromString: '23 Aug, 2003'.
	self assert: d start asString = '2003-08-23T00:00:00-04:00'.
	self assert: d duration asSeconds = (3600 * 24).

! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testDateLastSecondOfDayBeforeDstTransition
	"Check Date at the last second of the day before DST transition in right/America/Detroit."

	"(self new setTestSelector: #testDateLastSecondOfDayBeforeDstTransition) debug"

	| dt s |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	s := self localSmalltalkDstTransition - (2 * 3600) - 1.
	dt := Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-30'! !

!TimeTransformTest methodsFor: 'testing-Date' stamp: 'PaulDeBruicker 2/13/2016 17:19'!
testDateLastSecondOfDayBeforeDstTransitionNoLeapSeconds
	"Check Date at the last second of the day before DST transition in posix/America/Detroit."

	"(self new setTestSelector: #testDateLastSecondOfDayBeforeDstTransitionNoLeapSeconds) debug"

	| dt s |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	s := self localSmalltalkDstTransition - (2 * 3600) - 1.
	dt := Date fromSeconds: s.
	self assert: dt yyyymmdd = '2004-10-30'! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffset
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffset) debug"

	| secs dt offset expectedOffset pt |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	secs := 3273571904. "A time during summer 2004"

	pt := PointInTime fromSeconds: secs.
	dt := pt asDateAndTime.
	self assert: ('2004-09-2*' match: dt printString).
	offset := dt offset.
	expectedOffset := Duration days: 0 hours: -4 minutes: 0 seconds: 00.
	self assert: offset = expectedOffset.

	dt := DateAndTime fromLocalSeconds: secs.
	self assert: ('2004-09-2*' match: dt printString).
	offset := dt offset.
	self hasBrokenDateAndTimeConversion
		ifTrue: ["Offset possibly wrong, limitation of DateAndTime from local Smalltalk seconds"
				self assert: offset = DateAndTime now offset]
		ifFalse: [self assert: offset = expectedOffset]

! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffsetAtTransitionTime
	"Test the offset at the exact second of the DST transition time"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetAtTransitionTime) debug"

	| pt offsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds.
	offsetSeconds := LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffsetImmediatelyAfter
	"Test the offset during a time period when DST is no longer in effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetImmediatelyAfter) debug"

	| pt offsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + 1).
	offsetSeconds := LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffsetImmediatelyBefore
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetImmediatelyBefore) debug"

	| pt offsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - 1).
	offsetSeconds := LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-4 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffsetLongAfter
	"Test the offset during a time period when DST is no longer in effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetLongAfter) debug"

	| pt offsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds + (3600 * 24 * 7)).
	offsetSeconds := LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-5 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-TZ offset at DST transition' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDaylightSavingsTimeOffsetLongBefore
	"Test the offset during a time period when DST is effect"

	"(self new setTestSelector: #testDaylightSavingsTimeOffsetLongBefore) debug"

	| pt offsetSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	pt := PointInTime fromAbsoluteSeconds: (dstTransitionAbsoluteSeconds - (3600 * 24 * 7)).
	offsetSeconds := LocalTimeTransform here localOffsetSecondsAt: pt.
	self assert: offsetSeconds = (-4 * 3600).
	self assert: pt asDateAndTime offset asSeconds = offsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDurationAcrossDstTransition
	"Test DateAndTime>>- across a DST transition."

	"(self new setTestSelector: #testDurationAcrossDstTransition) debug"

	| ptMidnight ptAfterDstTransition dtMidnight dtAfterDstTransition duration secondsDifference durationSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	ptMidnight := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference := ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	dtMidnight := ptMidnight asDateAndTime.
	dtAfterDstTransition := ptAfterDstTransition asDateAndTime.

	self assert: (dtAfterDstTransition asPointInTime asPosixSeconds - dtMidnight asPointInTime asPosixSeconds) = (3600 * 8).

	duration := dtAfterDstTransition - dtMidnight.
	durationSeconds := duration asSeconds.
	self assert: durationSeconds = secondsDifference.

! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDurationForSamePointInTimeIsZero
	"The same PointInTime expressed as DateInTime for two different time zones should have
	a Duration difference of zero."

	"(self new setTestSelector: #testDurationForSamePointInTimeIsZero) debug"

	| aPointInTime est mdt estDateAndTime mdtDateAndTime duration |
	aPointInTime := PointInTime fromAbsoluteSeconds: 1100000000.
	est := TimeZoneDatabase systemDatabase timeZoneFor: self posixDetroitTimeZoneName.
	mdt := TimeZoneDatabase systemDatabase timeZoneFor: self posixPhoenixTimeZoneName.
	estDateAndTime := aPointInTime asDateAndTimeForTimeZone: est.
	mdtDateAndTime := aPointInTime asDateAndTimeForTimeZone: mdt.
	self assert: (estDateAndTime asPointInTime asAbsoluteSeconds = mdtDateAndTime asPointInTime asAbsoluteSeconds).
	duration := mdtDateAndTime - estDateAndTime.
	self assert: duration asSeconds = 0.

! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testDurationWithNoTransition
	"Just a normal duration, no DST or leap seconds to worry about. This tests the DateAndTime>>- method."

	"(self new setTestSelector: #testDurationWithNoTransition) debug"

	| ptMidnight ptAfterDstTransition dtMidnight dtAfterDstTransition duration secondsDifference durationSeconds anArbitrarySecondsCount |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	anArbitrarySecondsCount := 1100000000.
	ptMidnight := PointInTime fromAbsoluteSeconds: anArbitrarySecondsCount - 1.
	ptAfterDstTransition := PointInTime fromAbsoluteSeconds: anArbitrarySecondsCount + 1.
	secondsDifference := ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = 2.

	dtMidnight := ptMidnight asDateAndTime.
	dtAfterDstTransition := ptAfterDstTransition asDateAndTime.
	duration := dtAfterDstTransition - dtMidnight.
	durationSeconds := duration asSeconds.
	self assert: durationSeconds = secondsDifference.

! !

!TimeTransformTest methodsFor: 'testing-fileformat-3' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testIsraelTimeZoneChanges

"the Israel time zone file is in the version 3 format.  So we will just check if the DST transitions for 2016 are correct and assume everything in the version 3 file format processing is correct if they are.  This is just and adaptation of the #tesetPointInTimeAtDstTransition andtestPointInTimeImmediatelyAfter"

	"(self new setTestSelector: #testIsraelTimeZoneChanges) debug"

	| p springTransitionDSTPosixSeconds autumnTransitionDSTPosixSeconds |
	
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: 'Israel').
	TimeZoneDatabase systemDatabase defaultLocation: 'Israel'.

	springTransitionDSTPosixSeconds:= 1458863999.
	autumnTransitionDSTPosixSeconds:= 1477781999.


	p :=(PointInTime fromPosixSeconds: springTransitionDSTPosixSeconds )
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor:  'Israel').
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 3.
	self assert: p dayOfMonth==25.
	self assert: p year == 2016.
	self assert: p julianDayNumber =  2457473.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 2 * 60 * 60).
	self assert: p printString = '2016-03-25T01:59:59+02:00'.
	
	p :=(PointInTime fromPosixSeconds: springTransitionDSTPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'Israel').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 3.
	self assert: p month == 3.
	self assert: p dayOfMonth==25.
	self assert: p year == 2016.
	self assert: p julianDayNumber =  2457473.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 3 * 60 * 60).
	self assert: p printString = '2016-03-25T03:00:00+03:00'.


	p :=(PointInTime fromPosixSeconds: autumnTransitionDSTPosixSeconds )
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor:  'Israel').
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p dayOfMonth==30.
	self assert: p year == 2016.
	self assert: p julianDayNumber =    2457692.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 3 * 60 * 60).
	self assert: p printString = '2016-10-30T01:59:59+03:00'.
	
	p :=(PointInTime fromPosixSeconds: autumnTransitionDSTPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'Israel').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p dayOfMonth==30.
	self assert: p year == 2016.
	self assert: p julianDayNumber =   2457692.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 2 * 60 * 60).
	self assert: p printString = '2016-10-30T01:00:00+02:00'.
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testLeapSecondCount
	"Count of leap seconds as of 17 Oct, 2004 should be 22 for a time zone with leap
	second table. The UTC time zone does not have a leap second table, so the leap
	second count is zero."

	| tpSec leap |
	tpSec := 1098000000.

	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	leap := LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: ((PointInTime fromPosixSeconds: tpSec) asDateAndTime year) = 2004.
	self assert: leap == (self osUnderstandsLeapSeconds ifTrue:[22] ifFalse:[0]).

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	leap := LocalTimeTransform here leapSecondsAt: tpSec.
	self assert: ((PointInTime fromPosixSeconds: tpSec) asDateAndTime year) = 2004.
	self assert: leap == 0.

! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testLocalDetroitEpoch
	"Check DateAndTime for the beginning of the Posix epoch expressed as time in
	the local time zone (see #setUp)."

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
		self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p := (PointInTime fromPosixSeconds: 0) asDateAndTime.
	self assert: p julianDayNumber = 2440587.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p year = 1969.
	self assert: p month = 12.
	self assert: p hour = 19.
	self assert: p minute = 0.
	self assert: p second = 0! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testOffsetAsUtc
	"Check the right/America/Detroit transition time expressed as UTC (where UTC does not account for leap seconds)."

	"(self new setTestSelector: #testOffsetAsUtc) debug"
	self osUnderstandsLeapSeconds
		ifTrue: [
	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'UTC').
	"transition time should be '31 October 2004 6:00:22 am UTC'"
	self assert: p printString = '2004-10-31T06:00:22+00:00'.
	self assert: p offset = (Duration seconds: 0).]

! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testOffsetAsUtcWithLeapSeconds
	"Check the right/America/Detroit and posix/America/Detroit transition time expressed as UTC.
	The transition time should be '31 October 2004 6:00:00 UTC.'"

	"(self new setTestSelector: #testOffsetAsUtcWithLeapSeconds) debug"
	self osUnderstandsLeapSeconds
		ifTrue: [
	| p1 p2 dt |
	"Test with a leap second aware local time zone"
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p1 := PointInTime fromPosixSeconds: dstTransitionPosixSeconds.
	dt :=	p1 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightUTCTimeZoneName).
	self assert: dt printString = '2004-10-31T06:00:00+00:00'.
	self assert: dt offset = (Duration seconds: 0).
	dt :=	p1 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self posixUTCTimeZoneName).
	self assert: dt printString = '2004-10-31T06:00:22+00:00'.
	self assert: dt offset = (Duration seconds: 0).

	"Test with a local time zone that ignores leap seconds"
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	p2 := PointInTime fromPosixSeconds: dstTransitionPosixSeconds.
	self assert: (p1 absoluteTime - p2 absoluteTime) = 22.
	dt :=	p2 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self posixUTCTimeZoneName).
	self assert: dt printString = '2004-10-31T06:00:00+00:00'.
	self assert: dt offset = (Duration seconds: 0).
	dt :=	p2 asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightUTCTimeZoneName).
	self assert: dt printString = '2004-10-31T05:59:38+00:00'.
	self assert: dt offset = (Duration seconds: 0)].
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeAtDstTransition
	"Check DateAndTime at the first second of the Fall 2004 DST period in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeAtDstTransition) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeAtDstTransitionNoLeapSeconds
	"Check DateAndTime at the first second of the Fall 2004 DST period in America/Detroit."

	"(self new setTestSelector: #testPointInTimeAtDstTransitionNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeImmediatelyAfter
	"Check DateAndTime for the first second after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyAfter) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 1.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeImmediatelyAfterNoLeapSeconds
	"Check DateAndTime for the first second after the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyAfterNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 1.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:01-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeImmediatelyBefore
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyBefore) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeImmediatelyBeforeNoLeapSeconds
	"Check DateAndTime for the second immediately prior to the Fall 2004 DST transition in America/Detroit."

	"(self new setTestSelector: #testPointInTimeImmediatelyBeforeNoLeapSeconds) debug"

	| p |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 1)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'America/Detroit').
	self assert: p seconds == 59.
	self assert: p minutes == 59.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:59:59-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeNowSeconds
	"Check consistency of various views on the current time"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPointInTimeNowSeconds) debug"

	| clockSeconds pt1 pt2 ptSeconds1 ptSeconds2 pt3 pt4 ptSeconds3 ptSeconds4 |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	clockSeconds := Time totalLocalSeconds.
	pt1 := PointInTime now.
	pt2 := PointInTime fromSmalltalkSeconds: clockSeconds.
	ptSeconds1 := pt1 asLocalSmalltalkSeconds.
	ptSeconds2 := pt2 asLocalSmalltalkSeconds.
	self assert: (clockSeconds - ptSeconds1) abs < 1.
	self assert: (ptSeconds2 - ptSeconds1) abs < 1.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds := Time totalLocalSeconds.
	pt3 := PointInTime now.
	pt4 := PointInTime fromSmalltalkSeconds: clockSeconds.
	ptSeconds3 := pt3 asLocalSmalltalkSeconds.
	ptSeconds4 := pt4 asLocalSmalltalkSeconds.
	self assert: (clockSeconds - ptSeconds3) abs < 1.
	self assert: (ptSeconds4 - ptSeconds3) abs < 1.
	self assert: (ptSeconds4 - ptSeconds2) abs < 1.
	self assert: (ptSeconds3 - ptSeconds1) abs < 1! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeOneDayAfter
	"Check DateAndTime for 24 hours after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneDayAfter) debug"

	| p oneDay |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	oneDay := 3600 * 24.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + oneDay)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 11.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453311.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-11-01T01:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeOneDayBefore
	"Check DateAndTime for 24 hours prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneDayBefore) debug"

	| p oneDay |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	oneDay := 3600 * 24.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - oneDay)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 2.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453309.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-30T02:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeOneHalfHourAfter
	"Check DateAndTime for one half hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHalfHourAfter) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 1800)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 30.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T01:30:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeOneHourAfter
	"Check DateAndTime for one hour after the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHourAfter) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds + 3600)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 2.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -5 * 60 * 60).
	self assert: p printString = '2004-10-31T02:00:00-05:00'
! !

!TimeTransformTest methodsFor: 'testing-PointInTime TZ offset' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimeOneHourBefore
	"Check DateAndTime for one hour prior to the Fall 2004 DST transition in right/America/Detroit."

	"(self new setTestSelector: #testPointInTimeOneHourBefore) debug"

	| p |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	p :=	(PointInTime fromPosixSeconds: dstTransitionPosixSeconds - 3600)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName).
	self assert: p seconds == 0.
	self assert: p minutes == 0.
	self assert: p hours == 1.
	self assert: p month == 10.
	self assert: p year == 2004.
	self assert: p julianDayNumber = 2453310.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: -4 * 60 * 60).
	self assert: p printString = '2004-10-31T01:00:00-04:00'
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPointInTimePosixOffset
	"Verify difference in seconds between Posix epoch and Smalltalk epoch GMT."

	| tp ts diff |
	tp := PointInTime fromPosixSeconds: 0.
	ts := PointInTime fromUtcSmalltalkSeconds: 0.
	self assert: PointInTime posixOffset = self posixOffset.
	diff := tp asInteger - ts asInteger.
	self assert: self posixOffset = diff
! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPosixEpoch
	"Check DateAndTime for the beginning of the Posix epoch expressed as UTC."

	| p |
	p :=	(PointInTime fromPosixSeconds: 0)
		asDateAndTimeForTimeZone: (TimeZoneDatabase systemDatabase timeZoneFor: 'UTC').
	self assert: p seconds == 0.
	self assert: p julianDayNumber = 2440588.
	self assert: p nanoSecond == 0.
	self assert: p offset = (Duration seconds: 0)
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPosixSecondsVersusAtomicSeconds
	"Test elapsed atomic clock seconds versus elapsed Posix seconds."

	"(self new setTestSelector: #testPosixSecondsVersusAtomicSeconds) debug"

	| absoluteSeconds posixSeconds pt1 pt2 diff |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	absoluteSeconds := dstTransitionAbsoluteSeconds. "Atomic clock seconds since January 1, 1970 UTC"
	posixSeconds := dstTransitionPosixSeconds . "Posix clock seconds since January 1, 1970 UTC"
	pt1 := PointInTime fromAbsoluteSeconds: absoluteSeconds.
	pt2 := PointInTime fromPosixSeconds: posixSeconds.
	diff := pt1 - pt2.
	self assert: diff abs < 1.
	self assert: pt1 asAbsoluteSeconds = absoluteSeconds.
	self assert: pt2 asPosixSeconds = posixSeconds.

	absoluteSeconds := 0. "Atomic clock seconds at January 1, 1970 UTC"
	posixSeconds := 0. "Posix clock seconds at January 1, 1970 UTC"
	pt1 := PointInTime fromAbsoluteSeconds: absoluteSeconds.
	pt2 := PointInTime fromPosixSeconds: posixSeconds.
	diff := pt1 - pt2.
	self assert: diff abs < 1.
	self assert: pt1 asAbsoluteSeconds = absoluteSeconds.
	self assert: pt2 asPosixSeconds = posixSeconds.
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPosixToPointInTimeConversion
	"Posix seconds should differ from PointInTime seconds by the number of leap
	seconds since the Posix epoch."

	"(self new setTestSelector: #testPosixToPointInTimeConversion) debug"

	| ptFromPosix ptFromAbsolute ptSec |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	ptFromPosix := PointInTime fromPosixSeconds: dstTransitionPosixSeconds - (2 * 3600). "midnight"
	ptFromAbsolute := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (2 * 3600). "midnight"
	self assert: ptFromPosix = ptFromAbsolute.
	ptSec := ptFromAbsolute absoluteTime.
	self assert: 2 * 3600 + ptSec = dstTransitionAbsoluteSeconds
! !

!TimeTransformTest methodsFor: 'testing-Posix seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPosixToPointInTimeConversionNoLeapSeconds
	"Posix seconds should differ from PointInTime seconds by the number of leap
	seconds since the Posix epoch, zero for a time zone with no leap second table."

	| pt ptSec |
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	pt := PointInTime fromPosixSeconds: dstTransitionPosixSeconds - (2 * 3600). "midnight"
	ptSec := pt absoluteTime.
	self assert: (dstTransitionPosixSeconds - ptSec) = (2 * 3600)
! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPrimPosixTimeMicrosecondResolution
	"Make sure that local time is reported similarly with and without the plugin.
	The #assert: is only evaluated if the plugin is available, otherwise this test
	skips the #assert: passes."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPrimPosixTimeMicrosecondResolution) debug"

	| primSeconds expectedSeconds |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	primSeconds := PointInTime primPosixTimeMicrosecondResolution.
	expectedSeconds := Time totalLocalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalLocalSeconds).
	primSeconds ifNotNil: [ self assert: (primSeconds - expectedSeconds) abs < 1 ].
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	primSeconds := PointInTime primPosixTimeMicrosecondResolution.
	expectedSeconds := Time totalLocalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalLocalSeconds).
	primSeconds ifNotNil: [ self assert: (primSeconds - expectedSeconds) abs < 1 ]! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPrimPosixTimeMicrosecondResolutionOffset
	"(self new setTestSelector: #testPrimPosixTimeMicrosecondResolutionOffset) debug"

	| primSecs offset smalltalkSeconds |
	primSecs := PointInTime primPosixTimeMicrosecondResolution.
	primSecs
		ifNotNil: [ 
			offset := PointInTime primPosixTimeAndLocalOffset second.
			smalltalkSeconds := Time totalLocalSeconds.
			self assert: (primSecs - (smalltalkSeconds - PointInTime posixOffset + offset)) abs < 1 ]! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPrimSecondsClock
	"Compare the default Squeak clock prim seconds to what is reported by
	the plugin."

	"(self new setTestSelector: #testPrimSecondsClock) debug"

	| primSecondsClock primPosixTime secondsFromPosixTime diff offset |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: #'America/Detroit').
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	primSecondsClock := self primSecondsClock.
	(primPosixTime := PointInTime primPosixTimeMicrosecondResolution)
		ifNotNil: [ 
			secondsFromPosixTime := primPosixTime + PointInTime posixOffset.
			offset := self primPosixTimeAndLocalOffset last.
			diff := secondsFromPosixTime - primSecondsClock - offset.
			self assert: diff abs < 1 ]! !

!TimeTransformTest methodsFor: 'testing-time plugin' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testPrimVersusApproximateTimeNow
	"Compare the default Squeak clock prim seconds to what is reported by
	the plugin."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testPrimVersusApproximateTimeNow) debug"

	| approxTime primPosixTime diff |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	approxTime := Time totalLocalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalLocalSeconds).
	(primPosixTime := PointInTime primPosixTimeMicrosecondResolution)
		ifNotNil: [ 
			diff := primPosixTime - approxTime.
			self assert: diff abs < 1 ].
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	approxTime := Time totalLocalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalLocalSeconds).
	(primPosixTime := PointInTime primPosixTimeMicrosecondResolution)
		ifNotNil: [ 
			diff := primPosixTime - approxTime.
			self assert: diff abs < 1 ].
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	approxTime := Time totalLocalSeconds - PointInTime posixOffset
		- (LocalTimeTransform here localOffsetSecondsAt: Time totalLocalSeconds).
	(primPosixTime := PointInTime primPosixTimeMicrosecondResolution)
		ifNotNil: [ 
			diff := primPosixTime - approxTime.
			self assert: diff abs < 1 ]! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testSecondsCountAcrossDstTransition

	"(self new setTestSelector: #testSecondsCountAcrossDstTransition) debug"

	| ptMidnight ptAfterDstTransition secondsDifference localSecsMidnight localSecsAfterDstTransition |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	ptMidnight := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference := ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	localSecsMidnight := ptMidnight asLocalSmalltalkSeconds.
	localSecsAfterDstTransition := ptAfterDstTransition asLocalSmalltalkSeconds.
	self assert: (localSecsAfterDstTransition - localSecsMidnight) = (secondsDifference - 3600).
! !

!TimeTransformTest methodsFor: 'testing-Duration DST' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testSecondsCountAcrossDstTransitionNoLeapSeconds

	"(self new setTestSelector: #testSecondsCountAcrossDstTransitionNoLeapSeconds) debug"

	| ptMidnight ptAfterDstTransition secondsDifference localSecsMidnight localSecsAfterDstTransition |
	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	ptMidnight := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (3600 * 2).
	ptAfterDstTransition := PointInTime fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (3600 * 6).
	secondsDifference := ptAfterDstTransition asAbsoluteSeconds - ptMidnight asAbsoluteSeconds.
	self assert: secondsDifference = (3600 * 8).

	localSecsMidnight := ptMidnight asLocalSmalltalkSeconds.
	localSecsAfterDstTransition := ptAfterDstTransition asLocalSmalltalkSeconds.
	self assert: (localSecsAfterDstTransition - localSecsMidnight) = (secondsDifference - 3600).
! !

!TimeTransformTest methodsFor: 'testing-Duration leap seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testSecondsCountAcrossLeapSecondTransition
	"PointInTime seconds are atomic clock seconds. Posix seconds and Smalltalk
	seconds are both rotational seconds. Leap seconds are added so that rotational
	seconds counting stays aligned with the rotation of the Earth."

	"(self new setTestSelector: #testSecondsCountAcrossLeapSecondTransition) debug"
	self osUnderstandsLeapSeconds
		ifTrue: [
	| ptBefore ptAfter secondsDifference localSecsBefore localSecsAfter aLeapSecondTransitionTime posixSecsBefore posixSecsAfter |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	aLeapSecondTransitionTime := 915148821. "From the leap second table in right/America/Detroit"

	"Set up an absolute time duration of two seconds, spanning the leap second transition"
	ptBefore := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 1.
	ptAfter := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime + 1.
	secondsDifference := ptAfter asAbsoluteSeconds - ptBefore asAbsoluteSeconds.
	self assert: secondsDifference = 2.

	"Local seconds should count two absolute seconds as the same local second"
	localSecsBefore := ptBefore asLocalSmalltalkSeconds.
	localSecsAfter := ptAfter asLocalSmalltalkSeconds.
	self assert: (localSecsAfter - localSecsBefore) = 1.

	"Posix seconds also count two absolute seconds as the same Posix second"
	posixSecsBefore := ptBefore asPosixSeconds.
	posixSecsAfter := ptAfter asPosixSeconds.
	self assert: (posixSecsAfter - posixSecsBefore) = 1.]


! !

!TimeTransformTest methodsFor: 'testing-PointInTime seconds' stamp: 'PaulDeBruicker 3/3/2016 10:59'!
testSmalltalkEpoch
	"Confirm that Smalltalk seconds are interpreted as wall clock time in the
	local time zone."

	"(self new setTestSelector: #testSmalltalkEpoch) debug"

	| smalltalkOriginDetroit smalltalkOriginUtc difference currentDetroitOffsetSeconds |

	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	currentDetroitOffsetSeconds := DateAndTime now offset asSeconds.
	self assert: currentDetroitOffsetSeconds < 0. "ensure nonzero"
	smalltalkOriginDetroit := DateAndTime fromLocalSeconds: 0.

	TimeZoneDatabase systemDatabase defaultLocation: self posixUTCTimeZoneName.
	smalltalkOriginUtc := DateAndTime fromLocalSeconds: 0.

	self assert: smalltalkOriginDetroit > smalltalkOriginUtc.

	difference := smalltalkOriginUtc - smalltalkOriginDetroit.
	self assert: difference asSeconds = currentDetroitOffsetSeconds
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'dtl 11/6/2004 22:32'!
testSubtractDateAndTime
	"Test DateAndTime>>- "

	"(self new setTestSelector: #testSubtractDateAndTime) debug"

	| dtOne dtTwo twoMinusOne dtThree threeMinusOne dtFour dtFive fiveMinusFour dtSix dtSeven sevenMinusSix |
	"One day duration"
	dtOne := '2004-11-27T23:59:37-05:00' asDateAndTime.
	dtTwo := '2004-11-28T23:59:37-05:00' asDateAndTime.
	twoMinusOne := dtTwo - dtOne.
	self assert: twoMinusOne asSeconds = (24 * 3600).

	"dtThree is the same wall clock display, but one hour to the east, hence it represents one
	hour earlier in absolute time."
	dtThree := '2004-11-27T23:59:37-04:00' asDateAndTime.
	threeMinusOne := dtThree - dtOne.
	self assert: threeMinusOne asSeconds = -3600.

	"At the same real time, the wall clock shows one hour less as we move westward by one hour of Earth angle."
	dtFour := '2004-11-27T23:59:37-04:00' asDateAndTime.
	dtFive := '2004-11-27T22:59:37-05:00' asDateAndTime.
	fiveMinusFour := dtFive - dtFour.
	self assert: fiveMinusFour asSeconds = 0.

	dtSix := '2004-11-27T22:59:37-04:00' asDateAndTime.
	dtSeven := '2004-11-27T23:59:37-05:00' asDateAndTime.
	sevenMinusSix := dtSeven - dtSix.
	self assert: sevenMinusSix asSeconds = (3600 * 2).
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testSubtractDateAndTimeAcrossDstTransition

	"(self new setTestSelector: #testSubtractDateAndTimeAcrossDstTransition) debug"

	| ptMidnight ptMidnightFourHoursLater midnight fourHoursLater difference |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	ptMidnight := PointInTime
		fromAbsoluteSeconds: dstTransitionAbsoluteSeconds - (2 * 3600).
	ptMidnightFourHoursLater := PointInTime
		fromAbsoluteSeconds: dstTransitionAbsoluteSeconds + (2 * 3600).
	midnight := '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater := '2004-10-31T03:00:00-05:00' asDateAndTime.
	self assert: (ptMidnight asDateAndTime - midnight) asSeconds abs < 1.
	self assert: (ptMidnightFourHoursLater asDateAndTime - fourHoursLater) asSeconds abs < 1.
	difference := fourHoursLater - midnight.
	self assert: 4 * 3600 = difference asSeconds
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testSubtractDurationAcrossDstTransition

	"(self new setTestSelector: #testSubtractDurationAcrossDstTransition) debug"

	| midnight fourHoursLater duration |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	midnight := '2004-10-31T00:00:00-04:00' asDateAndTime.
	fourHoursLater := '2004-10-31T03:00:00-05:00' asDateAndTime.
	duration := Duration seconds: (4 * 3600).
	self assert: fourHoursLater - duration = midnight
! !

!TimeTransformTest methodsFor: 'testing-arithmetic' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testSubtractDurationAcrossLeapSecondTransition

	"(self new setTestSelector: #testSubtractDurationAcrossLeapSecondTransition) debug"
	self osUnderstandsLeapSeconds
		ifTrue: [
	| aLeapSecondTransitionTime ptFirst ptTenSecondsLater |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	aLeapSecondTransitionTime := 915148821. "From the leap second table in right/America/Detroit"
	ptFirst := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2.
	ptTenSecondsLater := PointInTime fromAbsoluteSeconds: aLeapSecondTransitionTime - 2 + 10.
	self assert: (ptTenSecondsLater asAbsoluteSeconds - ptFirst asAbsoluteSeconds) = 10.
	self assert: (ptTenSecondsLater asPosixSeconds - ptFirst asPosixSeconds) = 9.
	self assert: (ptTenSecondsLater asLocalSmalltalkSeconds - ptFirst asLocalSmalltalkSeconds) = 9.
	self assert: (ptTenSecondsLater asDateAndTime - (Duration seconds: 9)) = ptFirst asDateAndTime]
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testTimeAsDateAndTime

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testTimeAsDateAndTime) debug"

	| dtNow ptNow timeNow |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	timeNow := Time dateAndTimeNow.
	dtNow := DateAndTime now.
	ptNow := PointInTime now asDateAndTime.
	self assert: (dtNow - ptNow) asSeconds abs < 1.
	self assert: ptNow asDate asString = timeNow first asString. "#asString becaues start offsets may differ"
	self assert: (ptNow asTime asSeconds - timeNow last asSeconds) abs < 1.
	self assert: dtNow asDate asString = timeNow first asString.
	self assert: (dtNow asTime asSeconds - timeNow last asSeconds) abs < 1.
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testTimeAsDateAndTimeUTC

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testTimeAsDateAndTimeUTC) debug"

	| dtNow ptNow timeNow |
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	timeNow := Time dateAndTimeNow.
	dtNow := DateAndTime now.
	ptNow := PointInTime now asDateAndTime.
	self assert: (dtNow - ptNow) asSeconds abs < 1.
	self assert: ptNow asDate = timeNow first.
	self assert: (ptNow asTime asSeconds - timeNow last asSeconds) abs < 1.
	self assert: dtNow asDate = timeNow first.
	self assert: (dtNow asTime asSeconds - timeNow last asSeconds) abs < 1.
! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testTimeNow
	"Make sure DateAndTime>>now and Time>>now show the same time. Allow
	one second of variance in the test to account for difference in precision
	of DateAndTime seconds and the integer seconds clock."

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset."

	"(self new setTestSelector: #testTimeNow) debug"

	| dt t dtSecs tSecs diff |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	dt := DateAndTime now.
	t := Time now.
	dtSecs := (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs := (t hour * 3600) + (t minute * 60) + t second.
	diff := (dtSecs - tSecs) abs.
	self assert: diff < 1.
	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	dt := DateAndTime now.
	t := Time now.
	dtSecs := (dt hour * 3600) + (dt minute * 60) + dt second.
	tSecs := (t hour * 3600) + (t minute * 60) + t second.
	diff := (dtSecs - tSecs) abs.
	self assert: diff < 1! !

!TimeTransformTest methodsFor: 'testing-Time' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
testTimeNowSeconds
	"Check consistency of various views on the current time"

	"This test will FAIL if the TimePlugin is being used. TimePlugin reports the
	operating system view of Posix seconds, while TimeZoneDatabase calculates
	Posix seconds from local seconds using the time zone offset. This test may
	also fail intermittently if it is interrupted by the process scheduler between
	calls to obtain the current time; these failures may be disregarded."

	"(self new setTestSelector: #testTimeNowSeconds) debug"

	| clockSeconds dtArray dt offset dateSeconds timeSeconds dtArraySeconds pt ptSeconds delta |
	delta := 1.5. "Number of seconds of error to allow to accomodate scheduler jitter"
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	clockSeconds := Time totalSeconds.
	dtArray := Time dateAndTimeNow.
	dt := DateAndTime now.
	pt := PointInTime now.
	offset := LocalTimeTransform here localOffsetSecondsAt: PointInTime now.
	dateSeconds := dtArray first asSeconds + offset. "local squeak seconds clock"
	timeSeconds := dtArray last asSeconds.
	dtArraySeconds := (dateSeconds + timeSeconds - offset).
	ptSeconds := pt asLocalSmalltalkSeconds.
	self assert: (clockSeconds - dt asSeconds) abs < delta.
	self assert: (clockSeconds - dtArraySeconds) abs < delta.
	self assert: (clockSeconds - ptSeconds) abs < delta.
	self assert: clockSeconds = (DateAndTime fromSeconds: clockSeconds) asSeconds.
	self assert: clockSeconds = (PointInTime
		fromSmalltalkSeconds: clockSeconds) asLocalSmalltalkSeconds.

	TimeZoneDatabase systemDatabase defaultLocation: 'UTC'.
	clockSeconds := Time totalSeconds.
	dtArray := Time dateAndTimeNow.
	dt := DateAndTime now.
	pt := PointInTime now.
	offset := LocalTimeTransform here localOffsetSecondsAt: PointInTime now.
	dateSeconds := dtArray first asSeconds + offset. "local squeak seconds clock"
	timeSeconds := dtArray last asSeconds.
	dtArraySeconds := (dateSeconds + timeSeconds - offset).
	ptSeconds := pt asLocalSmalltalkSeconds.
	self assert: (clockSeconds - dt asSeconds) abs < delta.
	self assert: (clockSeconds - dtArraySeconds) abs < delta.
	self assert: (clockSeconds - ptSeconds) abs < delta.
	self assert: clockSeconds = (DateAndTime fromSeconds: clockSeconds) asSeconds.
	self assert: clockSeconds = (PointInTime
		fromSmalltalkSeconds: clockSeconds) asLocalSmalltalkSeconds.
! !

!TimeTransformTest methodsFor: 'testing-misc' stamp: 'pad 2/16/2016 11:18'!
testTimeZoneTransitionSecond
	"This test just makes sure that the time zone tables contain the expected
	transition seconds. If the test fails, then either the tables do not contain
	the expected transition times, or there is something wrong with the offset
	lookup. Neither of these things should ever happen, so you'll have to
	figure it out on your own ;-)"

	"self new testTimeZoneTransitionSecond"

	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	self assert: self absoluteDstTransition = (self osUnderstandsLeapSeconds ifTrue:[ 1099202422] ifFalse:[ 1099202400]).
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self assert: self posixDstTransition = 1099202400.

	TimeZoneDatabase systemDatabase defaultLocation: self posixDetroitTimeZoneName.
	self assert: self absoluteDstTransition = 1099202400.
	TimeZoneDatabase systemDatabase defaultLocation: 'America/Detroit'.
	self assert: self posixDstTransition = 1099202400.
! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'PaulDeBruicker 3/4/2016 00:27'!
testTransformADateAndATimeFromTimeZone
	"(self new setTestSelector: #testTransformADateAndATimeFromTimeZone) debug"

	| date time z1 z2 z3 a |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	date := '23 Nov, 2004' asDate.
	time := '12:00' asTime.
	z1 := TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName.
	z2 := TimeZoneDatabase systemDatabase timeZoneFor: self rightChicagoTimeZoneName.
	z3 := TimeZoneDatabase systemDatabase timeZoneFor: self rightLosAngelesTimeZoneName.
	a := z1 transformFromTimeZone: z2 date: date time: time.
	self assert: a first asString = date asString.
	self assert: a first start offset asSeconds = (-5 * 3600).
	self assert: a second asSeconds - time asSeconds = 3600.
	a := z1 transformFromTimeZone: z3 date: date time: time.
	self assert: a first asString = date asString.
	self assert: a first start offset asSeconds = (-5 * 3600).
	self assert: a second asSeconds - time asSeconds = (3 * 3600).
	a := z2 transformFromTimeZone: z3 date: date time: time.
	self assert: a first asString = date asString.
	self assert: a second asSeconds - time asSeconds = (2 * 3600)! !

!TimeTransformTest methodsFor: 'testing-Date and Time array' stamp: 'PaulDeBruicker 2/13/2016 17:05'!
testTransformADateAndATimeToTimeZone

	"(self new setTestSelector: #testTransformADateAndATimeToTimeZone) debug"

	| date time z1 z2 z3 a |
	"Ensure that zoneinfo tables with leap seconds have been loaded"
	self assert: (TimeZoneDatabase systemDatabase timeZoneNames
		includes: self rightDetroitTimeZoneName).
	TimeZoneDatabase systemDatabase defaultLocation: self rightDetroitTimeZoneName.
	date := '23 Nov, 2004' asDate.
	time := '12:00' asTime.
	z1 := TimeZoneDatabase systemDatabase timeZoneFor: self rightDetroitTimeZoneName.
	z2 := TimeZoneDatabase systemDatabase timeZoneFor: self rightChicagoTimeZoneName.
	z3 := TimeZoneDatabase systemDatabase timeZoneFor: self rightLosAngelesTimeZoneName.
	a := z1 transformDate: date time: time toTimeZone: z2.
	self assert: a first asString = date asString. "dates are nominally the same but have different start offsets"
	self assert: a first start offset asSeconds = ( -6 * 3600 ).
	self assert: (time asSeconds - a second asSeconds) = (1 * 3600).
	a := z1 transformDate: date time: time toTimeZone: z3.
	self assert: a first asString = date asString.
	self assert: a first start offset asSeconds = ( -8 * 3600 ).
	self assert: (time asSeconds - a second asSeconds) = (3 * 3600).
	a := z2 transformDate: date time: time toTimeZone: z3.
	self assert: a first asString = date asString.
	self assert: a first start offset asSeconds = ( -8 * 3600 ).
	self assert: (time asSeconds - a second asSeconds) = (2 * 3600).
	a := z2 transformDate: date time: time toTimeZone: z1.
	self assert: a first asString = date asString.
	self assert: a first start offset asSeconds = ( -5 * 3600 ).
	self assert: (time asSeconds - a second asSeconds) = ( -1 * 3600).
! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/6/2004 19:37'!
testTwoDateAndTimesTheSame

	"(self new setTestSelector: #testTwoDateAndTimesTheSame) debug"

	| dtOne dtTwo duration ptOne ptTwo |
	"At the same real time, the wall clock shows one hour less as we move westward by one hour of Earth angle."

	dtOne := '2004-11-27T22:00:00-04:00' asDateAndTime.
	dtTwo := '2004-11-27T21:00:00-05:00' asDateAndTime.

	duration := dtTwo - dtOne.
	self assert: duration asSeconds = 0.

	"Verify that they are the same point in time."
	ptOne := dtOne asPointInTime.
	ptTwo := dtTwo asPointInTime.
	self assert: ptOne asAbsoluteSeconds = ptTwo asAbsoluteSeconds
! !

!TimeTransformTest methodsFor: 'testing-Duration' stamp: 'dtl 11/6/2004 22:33'!
testTwoDateAndTimesWithDifferentOffset
	"Two DateAndTimes with the same wall clock display, but with different time zone
	offsets. The one with the larger offset corresponds to a more westerly location
	on the planet, and thus represents a PointInTime of greater magnitude."

	"(self new setTestSelector: #testTwoDateAndTimesWithDifferentOffset) debug"

	| dtOne dtTwo duration ptOne ptTwo |
	"dtTwo is the same wall clock time, but one hour to the west, hence is a time that occurs
	one hour later than dtOne."
	dtOne := '2004-11-27T22:00:00-04:00' asDateAndTime.
	dtTwo := '2004-11-27T22:00:00-05:00' asDateAndTime.

	ptOne := dtOne asPointInTime.
	ptTwo := dtTwo asPointInTime.
	self assert: (ptTwo asAbsoluteSeconds - ptOne asAbsoluteSeconds) = 3600.

	duration := dtTwo - dtOne.
	self assert: duration asSeconds = 3600.
! !

!TimeTransformTest methodsFor: 'documentation' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
utcClockInfo

	"(StringHolder new acceptContents: self new utcClockInfo; yourself)
		openLabel: 'UTC seconds clock' "

	^ PointInTime utcClockInfo
! !

!DateAndTime class methodsFor: '*time-TimeZones' stamp: 'PaulDeBruicker 3/3/2016 15:52'!
fromLocalSeconds: seconds offsetSeconds: offset
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901"

	"Pharo only uses the offset for printing.  DateAndTime are stored as UTC internally. So, we include the offset"

	| since offsetDuration |
	since := Duration
		days: SqueakEpoch
		hours: 0
		minutes: 0
		seconds: seconds.
	offsetDuration := Duration seconds: offset.
	^ self basicNew
		ticks: since ticks offset: offsetDuration;
		yourself! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'PaulDeBruicker 3/14/2016 08:35'!
absoluteSeconds
	^ self asPosixSeconds! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'PaulDeBruicker 3/13/2016 16:27'!
asDateTime
	"DateAndTime now asPointInTime"

	^ DateTime fromDateAndTime: self! !

!DateAndTime methodsFor: '*time-UTC' stamp: 'dtl 11/10/2004 05:43'!
asPointInTime

	"DateAndTime now asPointInTime"

	^ self asPointInTimeInTimeZone: LocalTimeZone! !

DateAndTime subclass: #DateTime
	instanceVariableNames: 'timeZoneName'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-UTC'!

!DateTime commentStamp: 'PaulDeBruicker 3/13/2016 13:06' prior: 0!
DateTime stores the timeZoneName of the DateAndTime so we can ensure we are using the correct offset even after a legislative body changes when  those changes should occur.  

When created the offset is nil because we only really need it when printing.  Because of that we override DateAndTime methods that directly access #offset.  And prevent setting it from outside the instance


the offset is recalculated when the systems TimeZoneDatabase rules are updated from the zoneinfo files.!

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 6/5/2016 02:02'!
date: aDate time: aTime timeZone: aTimeZoneOrName
	^ self
		year: aDate year
		month: aDate monthName
		day: aDate dayOfMonth
		hour: aTime hour
		minute: aTime minute
		second: aTime second
		nanoSecond: aTime nanoSecond
		timeZone: aTimeZoneOrName! !

!DateTime class methodsFor: 'private' stamp: 'dtl 2/24/2019 16:57'!
dateError
	"Pharo has DateError as a subclass of Error"
	^(Smalltalk hasClassNamed: #DateError)
		ifTrue: [Smalltalk at: #DateError]
		ifFalse: [Error]
! !

!DateTime class methodsFor: 'instance creation queries' stamp: 'PaulDeBruicker 6/4/2016 13:13'!
epoch
	^ self
		year: 1901
		month: 1
		day: 1
		timeZone: #UTC! !

!DateTime class methodsFor: 'private' stamp: 'PaulDeBruicker 3/13/2016 14:06'!
findOffsetIn: aTimeZone onJdn: julianDayNumber withSeconds: localSeconds andNanos: nanoCount
	| posixSeconds offset1 offset2 |
	posixSeconds := (julianDayNumber - 2440588) * 86400 + localSeconds + (nanoCount / 1e9).
	offset1 := aTimeZone localOffsetSecondsAt: posixSeconds.
	offset2 := aTimeZone localOffsetSecondsAt: posixSeconds - offset1.
	^ offset2! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 16:31'!
fromDateAndTime: aDateAndTime
	| tz |
	tz := TimeZoneDatabase systemDatabase
		timeZoneWithOffset: aDateAndTime offset
		atPosixSeconds: aDateAndTime asPosixSeconds.
	^ self
		year: aDateAndTime year
		month: aDateAndTime month
		day: aDateAndTime dayOfMonth
		hour: aDateAndTime hour
		minute: aDateAndTime minute
		second: aDateAndTime second
		nanoSecond: aDateAndTime nanoSecond
		timeZone: tz! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 16:18'!
fromSeconds: utcSecondsSinceEpoch
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901 for the given timeZone"

	^ self fromSeconds: utcSecondsSinceEpoch inTimeZoneNamed: TimeZoneDatabase systemDatabase defaultLocation! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 13:13'!
fromSeconds: utcSecondsSinceEpoch inTimeZoneNamed: aTimeZoneName
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901 for the given timeZone"

	| integerSeconds nanos |
	integerSeconds := utcSecondsSinceEpoch truncated.
	nanos := integerSeconds = utcSecondsSinceEpoch
		ifTrue: [ 0 ]
		ifFalse: [ ((utcSecondsSinceEpoch - integerSeconds) * NanosInSecond) asInteger ].
	^ self basicNew
		ticks:
			{SqueakEpoch.
			integerSeconds.
			nanos}
		timeZoneName: aTimeZoneName! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 16:24'!
fromSeconds: utcSecondsSinceEpoch offset: aDuration
	"Answer a DateAndTime since the Squeak epoch: 1 January 1901 for the given timeZone"

	self shouldNotImplement! !

!DateTime class methodsFor: 'private' stamp: 'PaulDeBruicker 3/13/2016 12:37'!
posixOffset
	"Logic is lifted from Ian's Unix support code.
	Squeak epoc	h is Jan 1, 1901.  Unix epoch is Jan 1, 1970: 17 leap years
     and 52 non-leap years later than Squeak."

	"52 * 365 + (17 * 366) * 24 * 60 * 60"

	^ 2177452800! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/15/2016 10:14'!
readFrom: aStream
	"Parse and return a new DateAndTime instance from stream, 
	as a Date, an optional Time and an optional TimeZone offset.
	The time defaults to midnight, the timezone to the local offset"

	"self readFrom: '2013-03-04T23:47:52.876+01:00' readStream"

	| date time  |
	date := Date readFrom: aStream.
	[ aStream atEnd or: [ '0123456789Z+-' includes: aStream peek ] ] whileFalse: [ aStream next ].
	('0123456789' includes: aStream peek)
		ifTrue: [ time := Time readFrom: aStream ]
		ifFalse: [ time := Time midnight ].
	^ self
		year: date year
		month: date monthIndex
		day: date dayOfMonth
		hour: time hour
		minute: time minute
		second: time second
		nanoSecond: time nanoSecond! !

!DateTime class methodsFor: 'instance creation queries' stamp: 'PaulDeBruicker 3/13/2016 16:26'!
tomorrow
	^ self today asDate next asDateTime! !

!DateTime class methodsFor: 'instance creation queries' stamp: 'PaulDeBruicker 6/4/2016 13:13'!
unixEpoch
	^ self
		year: 1970
		month: 1
		day: 1
		timeZone: #UTC! !

!DateTime class methodsFor: 'instance-creation' stamp: 'PaulDeBruicker 6/10/2016 08:38'!
year: year day: dayOfYear hour: hour minute: minute second: second timeZone: timezone
	| y d |
	y := self
		year: year
		month: 1
		day: 1
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		timeZone: timezone.
	d := Duration days: dayOfYear - 1.
	^ y + d! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 13:11'!
year: year month: month day: day
	^ self
		year: year
		month: month
		day: day
		hour: 0
		minute: 0
		second: 0
		nanoSecond: 0
		timeZone: TimeZoneDatabase systemDatabase defaultTimeZone! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 13:11'!
year: year month: month day: day hour: hour minute: minute second: second
	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		timeZone: TimeZoneDatabase systemDatabase defaultTimeZone! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 3/13/2016 13:12'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount
	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		nanoSecond: nanoCount
		timeZone: TimeZoneDatabase systemDatabase defaultTimeZone! !

!DateTime class methodsFor: 'instance creation' stamp: 'dtl 2/24/2019 16:58'!
year: year month: month day: day hour: hour minute: minute second: second nanoSecond: nanoCount timeZone: aTimeZoneOrName
	"Return a DateAndTime with the values in the given time zone"

	| monthIndex daysInMonth p q r s julianDayNumber localSeconds utcSeconds offset tz |
	tz:= aTimeZoneOrName isSymbol 
				ifTrue:[TimeZoneDatabase systemDatabase timeZoneFor:  aTimeZoneOrName] 				ifFalse:[aTimeZoneOrName].
	monthIndex := month isInteger
		ifTrue: [ month ]
		ifFalse: [ 
			month isMonth
				ifTrue: [ month index ]
				ifFalse: [ Month indexOfMonth: month ] ].
	(monthIndex between: 1 and: 12)
		ifFalse: [ self dateError signal: 'There is no ' , monthIndex printString , 'th month' ].
	daysInMonth := Month daysInMonth: monthIndex forYear: year.
	day < 1
		ifTrue: [ self dateError signal: 'day may not be zero or negative' ].
	day > daysInMonth
		ifTrue: [ self dateError signal: 'day is after month ends' ].
	p := monthIndex - 14 quo: 12.
	q := year + 4800 + p.
	r := monthIndex - 2 - (12 * p).
	s := year + 4900 + p quo: 100.
	julianDayNumber := (1461 * q quo: 4) + (367 * r quo: 12) - (3 * s quo: 4) + (day - 32075).
	localSeconds := (hour * 60 + minute) * 60 + second.
	offset := self
		findOffsetIn: tz
		onJdn: julianDayNumber
		withSeconds: localSeconds
		andNanos: nanoCount.
	utcSeconds := localSeconds - offset.
	^ self basicNew
		setJdn: julianDayNumber
			seconds: utcSeconds
			nano: nanoCount
			offset: nil;
		timeZoneName: tz timeZoneName asSymbol;
		yourself! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 6/5/2016 02:01'!
year: year month: month day: day hour: hour minute: minute second: second timeZone: aTimeZoneOrName
	^ self
		year: year
		month: month
		day: day
		hour: hour
		minute: minute
		second: second
		nanoSecond: 0
		timeZone: aTimeZoneOrName! !

!DateTime class methodsFor: 'instance creation' stamp: 'PaulDeBruicker 6/5/2016 02:02'!
year: year month: month day: day timeZone: aTimeZoneOrName
	^ self
		year: year
		month: month
		day: day
		hour: 0
		minute: 0
		second: 0
		nanoSecond: 0
		timeZone: aTimeZoneOrName! !

!DateTime class methodsFor: 'instance creation queries' stamp: 'PaulDeBruicker 3/15/2016 02:37'!
yesterday
	^ self today asDate previous asDateTime! !

!DateTime methodsFor: 'arithmetic' stamp: 'PaulDeBruicker 3/15/2016 03:00'!
+ operand
	"operand conforms to protocol Duration"

	| durationTicks |
	durationTicks := operand asDuration ticks.
	^ self class basicNew
		setJdn: julianDayNumber + durationTicks first
			seconds: seconds + durationTicks second
			nano: nanos + durationTicks third
			offset: nil;
		timeZoneName: timeZoneName;
		yourself! !

!DateTime methodsFor: 'converting' stamp: 'PaulDeBruicker 4/26/2016 02:39'!
asDateTime
	^ self! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/15/2016 04:17'!
asUTC
	^ timeZoneName = #UTC ifTrue:[^self copy.]ifFalse:[^self >> #UTC]! !

!DateTime methodsFor: 'converting' stamp: 'PaulDeBruicker 3/15/2016 04:22'!
asUnixTime
	^ self asPosixSeconds asInteger! !

!DateTime methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/13/2016 11:52'!
inTimeZoneNamed: tzName
	"DateAndTime now inTimeZoneNamed: #CET "

	"DateAndTime now inTimeZoneNamed: #PST8PDT "

	"DateAndTime now inTimeZoneNamed: 'Australia/Perth' "
timeZoneName:=tzName.
	^ self asPointInTime asDateAndTimeForTimeZoneNamed: tzName! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/13/2016 12:53'!
localSecondsSinceEpoch
	^ self asSeconds + self offset asSeconds! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 6/4/2016 13:13'!
midnight
	"Answer a DateAndTime starting at midnight (towards the end of the day) local time"

	self
		dayMonthYearDo: [ :day :month :year | 
			^ self class
				year: year
				month: month
				day: day
				timeZone: timeZoneName ]! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 6/4/2016 13:06'!
noon
	"Answer a DateAndTime starting at noon"

	^ self
		dayMonthYearDo: [ :d :m :y | 
			self class
				year: y
				month: m
				day: d
				hour: 12
				minute: 0
				second: 0
				timeZone: timeZoneName ]! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/15/2016 04:14'!
offset
	^ offset
		ifNil: [ 
			| tz |
			tz := self timeZone.
			offset := tz isNil
				ifTrue: [ 0 seconds ]
				ifFalse: [ (tz localOffsetSecondsAt: self asPosixSeconds) seconds ] ]! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/13/2016 13:06'!
offset: anObject
	self shouldNotImplement! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/14/2016 07:52'!
printOn: aStream withLeadingSpace: printLeadingSpaceToo
	"Print as per ISO 8601 sections 5.3.3 and 5.4.1.
	If printLeadingSpaceToo is false, prints either:
		'YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	If printLeadingSpaceToo is true, prints either:
		' YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for positive years) or '-YYYY-MM-DDThh:mm:ss.s+ZZ:zz:z' (for negative years)
	"

	self printYMDOn: aStream withLeadingSpace: printLeadingSpaceToo.
	aStream nextPut: $T.
	self printHMSOn: aStream.
	nanos ~= 0
		ifTrue: [ 
			| n len |
			n := nanos.
			len := 9.
			[ n \\ 10 = 0 ]
				whileTrue: [ 
					n := n / 10.
					len := len - 1 ].
			aStream nextPut: $..
			n
				printOn: aStream
				base: 10
				length: len
				padded: true ].
	aStream
		nextPut:
			(self offset positive
				ifTrue: [ $+ ]
				ifFalse: [ $- ]).
	self offset hours abs
		printOn: aStream
		base: 10
		length: 2
		padded: true.
	aStream nextPut: $:.
	self offset minutes abs
		printOn: aStream
		base: 10
		length: 2
		padded: true.
	self offset seconds = 0
		ifFalse: [ 
			aStream
				nextPut: $:;
				print: self offset seconds abs truncated ]! !

!DateTime methodsFor: 'as yet unclassified' stamp: 'PaulDeBruicker 3/14/2016 07:52'!
resetOffset
	offset := nil! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/13/2016 12:56'!
ticks: ticks offset: utcOffset
	"we don't let people set the offset as a duration because the time zone name is more important.  WE can look up the offset."

	self shouldNotImplement! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/15/2016 02:53'!
ticks: ticks timeZoneName: aTimeZoneName
	"ticks is {julianDayNumber. secondCount. nanoSeconds}"

	self
		setJdn: (ticks at: 1)
			seconds: (ticks at: 2)
			nano: (ticks at: 3)
			offset: nil; 
		timeZoneName: aTimeZoneName! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 6/9/2016 12:28'!
timeZone
	timeZoneName isSymbol ifTrue:[
	^ TimeZoneDatabase systemDatabase timeZoneFor: timeZoneName] ifFalse:[^timeZoneName]! !

!DateTime methodsFor: 'accessing' stamp: 'PaulDeBruicker 3/13/2016 12:18'!
timeZoneName: aString 
	timeZoneName := aString! !

!DateTime methodsFor: 'updating' stamp: 'PaulDeBruicker 3/14/2016 08:45'!
update: aParameter
	"If a new system time zone database has been created, reset the offset so the new rules are adopted"

	"this may be a bad idea because the work to update the offsets for all instances may never be repaid and if historical times change maybe that shouldn't happen without some kind of human intervention.  "

	aParameter == #timeZoneDatabaseForSystem
		ifTrue: [ self resetOffset ]! !

Error subclass: #TimeZoneError
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-UTC'!

!LocalTimeTransform class methodsFor: '*Time-TimeZone concrete subclasses' stamp: 'dtl 2/24/2019 18:49'!
forOSServices
	"Answer the concrete class to use to provide time zone information from
	the underlying operating system. This approach is suitable for many systems
	and applications, and can provide good time zone information without the
	need to maintain a full TimeZoneDatabase in the image."

	(TimeZoneDatabase platform == #Squeak) ifTrue: [^ OSTimeZone].
	(TimeZoneDatabase platform == #VisualWorks) ifTrue: [^ VwOSTimeZone].
	(TimeZoneDatabase platform == #'Smalltalk/X') ifTrue: [^ StxOSTimeZone].
	^ OSTimeZone! !

!LocalTimeTransform class methodsFor: '*Time-TimeZone concrete subclasses' stamp: 'dtl 12/28/1999 20:01'!
forSmalltalk

	^ NaiveTimeZone! !

!LocalTimeTransform class methodsFor: '*Time-TimeZones instance creation' stamp: 'dtl 12/30/1999 12:07'!
hereUsingDatabase

	"LocalTimeTransform hereUsingDatabase"

	^ self forTzDatabase here! !

!LocalTimeTransform class methodsFor: '*Time-TimeZones instance creation' stamp: 'dtl 12/30/1999 12:07'!
hereUsingNothing

	"LocalTimeTransform hereUsingNothing"

	^ self forSmalltalk here! !

!LocalTimeTransform class methodsFor: '*Time-TimeZones instance creation' stamp: 'dtl 12/30/1999 12:08'!
hereUsingOSServices

	"LocalTimeTransform hereUsingOSServices"

	^ self forOSServices here! !

!LocalTimeTransform methodsFor: '*Time-TimeZones' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
leapSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	"LocalTimeTransform here leapSecondsForDate: Date today time: Time now"

	| seconds t |
	seconds := aDate asSeconds + aTime asSeconds.
	t := PointInTime fromSmalltalkSeconds: seconds inTimeZone: self.
	^ self leapSecondsAt: t
! !

!LocalTimeTransform methodsFor: '*Time-TimeZones' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
localOffsetSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	"LocalTimeTransform here localOffsetSecondsForDate: Date today time: Time now"

	| seconds t |
	seconds := aDate asSeconds + aTime asSeconds.
	t := PointInTime fromSmalltalkSeconds: seconds inTimeZone: self.
	^ self localOffsetSecondsAt: t
! !

!LocalTimeTransform methodsFor: '*Time-TimeZones' stamp: 'dtl 2/25/2019 08:18'!
transformDate: aDate time: aTime toTimeZone: aTimeZone
	"For aDate and aTime here, answer the corresponding date and time in aTimeZone."

	"(LocalTimeTransform for: 'America/Detroit')
		transformDate: Date today
		time: Time now
		toTimeZone: (LocalTimeTransform for: 'America/Los_Angeles')"

	^ (PointInTime date: aDate time: aTime timeZone: self) asDateAndTimeArrayForTimeZone: aTimeZone
! !

!LocalTimeTransform methodsFor: '*Time-TimeZones' stamp: 'dtl 2/25/2019 08:18'!
transformFromTimeZone: aTimeZone date: aDate time: aTime
	"For aDate and aTime in aTimeZone, answer the corresponding date and time here."

	"(LocalTimeTransform for: 'America/Detroit')
		transformFromTimeZone: (LocalTimeTransform for: 'America/Los_Angeles')
		date: Date today
		time: Time now"

	^ (PointInTime date: aDate time: aTime timeZone: aTimeZone) asDateAndTimeArrayForTimeZone: self! !

LocalTimeTransform subclass: #NaiveTimeZone
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!NaiveTimeZone commentStamp: '<historical>' prior: 0!
I represent a timezone with no rules. I can be used by simple systems which
believe themselves to be at the center of the universe, and which have no
interest in the details of time coordinate transformations.!

!NaiveTimeZone class methodsFor: 'instance creation' stamp: 'dtl 12/30/1999 11:56'!
here

	"NaiveTimeZone here"

	^ self new! !

!NaiveTimeZone class methodsFor: 'instance creation' stamp: 'dtl 12/28/1999 20:01'!
new

	^ super new timeZoneName: #Smalltalk! !

!NaiveTimeZone methodsFor: 'leap seconds' stamp: 'dtl 12/28/1999 20:01'!
leapSecondsAt: aPointInTime
	"Answer the number of leap seconds since January 1, 1970 UTC as of aPointInTime"

	^ 0! !

!NaiveTimeZone methodsFor: 'offsets from UTC' stamp: 'dtl 12/28/1999 20:01'!
localOffsetSecondsAt: aPointInTime
	"Answer the offset from UTC in this time zone as of aPointInTime"

	^ 0
! !

!NaiveTimeZone methodsFor: 'offsets from UTC' stamp: 'dtl 12/28/1999 20:01'!
localOffsetSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	^ 0! !

!NaiveTimeZone methodsFor: 'printing' stamp: 'dtl 12/28/1999 20:01'!
printOn: aStream

	super printOn: aStream.
	aStream nextPutAll: ' for ', timeZoneName! !

LocalTimeTransform subclass: #OSTimeZone
	instanceVariableNames: 'secondaryTimeZoneName daylightSavingsInEffect currentLocalOffset'
	classVariableNames: 'DefaultDaylightSavingsInEffect DefaultLocalOffsetSeconds DefaultSecondaryTimeZoneName DefaultTimeZoneName'
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!OSTimeZone commentStamp: '<historical>' prior: 0!
I represent a time zone for a location in the world. I know how to make inquiries to
the operating system to determine where I am and what my timezone rules should be.!

!OSTimeZone class methodsFor: 'instance creation' stamp: 'dtl 12/30/1999 11:55'!
here
	"Answer an instance representing the timezone at this location."

	"OSTimeZone here"

	^ super new initialize! !

!OSTimeZone class methodsFor: 'initialize-release' stamp: 'dtl 12/28/1999 20:01'!
initialize
	"Eastern Standard Time, Detroit, Michigan, USA"

	"OSTimeZone initialize"

	DefaultLocalOffsetSeconds := 18000.
	DefaultTimeZoneName := #EST.
	DefaultSecondaryTimeZoneName := #DST.
	DefaultDaylightSavingsInEffect := true! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
currentLocalOffset
	"Always update from the primitive if possible."

	^ currentLocalOffset := self setCurrentLocalOffset
! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
currentLocalOffset: anInteger
	"Number of seconds offset from UTC at the present time. Note that
	value changes as a function of the current time, for example when
	daylight savings time takes effect."

	currentLocalOffset := anInteger
! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
daylightSavingsInEffect
	"Always update from the primitive if possible."

	^ daylightSavingsInEffect := self setDaylightSavingsInEffect
! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
daylightSavingsInEffect: aBoolean

	daylightSavingsInEffect := aBoolean! !

!OSTimeZone methodsFor: 'initialize-release' stamp: 'dtl 12/28/1999 20:01'!
initialize

	self setTimeZoneName.
	self setSecondaryTimeZoneName.
	self setCurrentLocalOffset.
	self setDaylightSavingsInEffect
! !

!OSTimeZone methodsFor: 'leap seconds' stamp: 'dtl 12/28/1999 20:01'!
leapSecondsAt: aPointInTime
	"Answer the number of leap seconds since January 1, 1970 UTC as of aPointInTime.
	Keep it simple, ignore leap seconds and just answer 0."

	^ 0
! !

!OSTimeZone methodsFor: 'offsets from UTC' stamp: 'dtl 1/8/2000 13:03'!
localOffsetSecondsAt: aPointInTime
	"Answer the offset from UTC in this time zone as of aPointInTime"

	"OSTimeZone here localOffsetSecondsAt: PointInTime now"

	| t |
	t := aPointInTime asInteger.
	t isNil ifTrue: [^ nil].
	^ (self primLocalOffsetSecondsAt: t) negated! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetDaylightFlag
	"Answer true if daylight savings is in effect"

	<primitive: 'primitiveGetDaylightFlag' module: 'TimeFunctions'>
	^ DefaultDaylightSavingsInEffect! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetDaylightFlagAt: aPosixTimeExpressedAsFloat
	"Answer true if daylight savings is in effect"

	<primitive: 'primitiveGetDaylightFlagAt' module: 'TimeFunctions'>
	^ DefaultDaylightSavingsInEffect! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetTimeZone
	"Answer a four element array of two time zone names (primary and secondary),
	daylight savings time flag, and seconds offset from UTC, for the current
	time zone at the present time."

	<primitive: 'primitiveGetTimeZone' module: 'TimeFunctions'>
	^ nil! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetTimeZoneAt: aPosixTimeExpressedAsFloat
	"Answer a four element array of two time zone names (primary and secondary),
	daylight savings time flag, and seconds offset from UTC, for the current
	time zone at the indicated time."

	<primitive: 'primitiveGetTimeZoneAt' module: 'TimeFunctions'>
	^ nil! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetTimeZoneName
	"Answer time zone name"

	<primitive: 'primitiveGetTimeZoneName' module: 'TimeFunctions'>
	^ DefaultTimeZoneName! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primGetTimeZoneSecondaryName
	"Answer secondary time zone name"

	<primitive: 'primitiveGetTimeZoneSecondaryName' module: 'TimeFunctions'>
	^ DefaultSecondaryTimeZoneName! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primLocalOffsetSeconds

	<primitive: 'primitiveLocalOffsetSeconds' module: 'TimeFunctions'>
	^ DefaultLocalOffsetSeconds! !

!OSTimeZone methodsFor: 'primitives' stamp: 'dtl 12/28/1999 20:01'!
primLocalOffsetSecondsAt: aFloat

	<primitive: 'primitiveLocalOffsetSecondsAt' module: 'TimeFunctions'>
	^ DefaultLocalOffsetSeconds! !

!OSTimeZone methodsFor: 'printing' stamp: 'dtl 12/30/1999 19:03'!
printOn: aStream

	super printOn: aStream.
	timeZoneName isNil ifFalse: [
		aStream nextPut: $ .
		timeZoneName do: [:e | aStream nextPut: e]].
	secondaryTimeZoneName isNil ifFalse: [
		aStream nextPut: $ .
		secondaryTimeZoneName do: [:e | aStream nextPut: e]]
! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
secondaryTimeZoneName

	^ secondaryTimeZoneName! !

!OSTimeZone methodsFor: 'accessing' stamp: 'dtl 12/28/1999 20:01'!
secondaryTimeZoneName: aSymbol

	secondaryTimeZoneName := aSymbol! !

!OSTimeZone methodsFor: 'private' stamp: 'dtl 12/28/1999 20:01'!
setCurrentLocalOffset

	| offset |
	offset := self primLocalOffsetSeconds.
	offset notNil
		ifTrue: [^ currentLocalOffset := offset]
		ifFalse: [^ currentLocalOffset := DefaultLocalOffsetSeconds]
! !

!OSTimeZone methodsFor: 'private' stamp: 'dtl 12/28/1999 20:01'!
setDaylightSavingsInEffect

	| dst |
	dst := self primGetDaylightFlag.
	dst notNil
		ifTrue: [^ daylightSavingsInEffect := dst]
		ifFalse: [^ daylightSavingsInEffect := DefaultDaylightSavingsInEffect]
! !

!OSTimeZone methodsFor: 'private' stamp: 'dtl 12/28/1999 20:01'!
setSecondaryTimeZoneName

	| tz |
	tz := self primGetTimeZoneSecondaryName.
	tz notNil
		ifTrue: [^ secondaryTimeZoneName := tz]
		ifFalse: [^ secondaryTimeZoneName := DefaultSecondaryTimeZoneName]! !

!OSTimeZone methodsFor: 'private' stamp: 'dtl 12/28/1999 20:01'!
setTimeZoneName

	| tz |
	tz := self primGetTimeZoneName.
	tz notNil
		ifTrue: [^ timeZoneName := tz]
		ifFalse: [^ timeZoneName := DefaultTimeZoneName]! !

LocalTimeTransform subclass: #StxOSTimeZone
	instanceVariableNames: 'timeZone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!StxOSTimeZone commentStamp: '<historical>' prior: 0!
I provide access to time zone functions provided by the base Smalltalk/X system.
!

!StxOSTimeZone class methodsFor: 'instance creation' stamp: ''!
here
	"Answer the current time zone, or nil if the classes have not been initialized for this system."

	"StxOSTimeZone here"

	^ super new initialize! !

!StxOSTimeZone methodsFor: 'initialize' stamp: ''!
initialize

        self timeZoneName
! !

!StxOSTimeZone methodsFor: 'leap seconds' stamp: ''!
leapSecondsAt: aPointInTime
	"Answer the number of leap seconds since January 1, 1970 UTC as of aPointInTime.
	Keep it simple, ignore leap seconds and just answer 0."

	^ 0! !

!StxOSTimeZone methodsFor: 'leap seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
leapSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	"StxOSTimeZone here leapSecondsForDate: Date today time: Time now"

	| seconds t |
	seconds := aDate asSeconds + aTime asSeconds.
	t := PointInTime fromSmalltalkSeconds: seconds inTimeZone: self.
	^ self leapSecondsAt: t! !

!StxOSTimeZone methodsFor: 'offsets from UTC' stamp: ''!
localOffsetSecondsAt: aPointInTime
        "Answer the offset from UTC in this time zone as of aPointInTime"

        "StxOSTimeZone here localOffsetSecondsAt: PointInTime now"

        ^ ((Smalltalk at: #AbsoluteTime) now) utcOffset negated

! !

!StxOSTimeZone methodsFor: 'offsets from UTC' stamp: ''!
localOffsetSecondsForDate: aDate time: aTime
        "Answer the offset from UTC in this time zone as of aDate and aTime, where
        aDate and aTime are in the context of this time zone."

        "StxOSTimeZone here localOffsetSecondsForDate: Date today time: Time now"

        ^ ((Smalltalk at: #AbsoluteTime) fromDate:aDate andTime:aTime) utcOffset negated
! !

!StxOSTimeZone methodsFor: 'accessing' stamp: ''!
timeZoneName
	"I do not know my name, so just answer the name of my class."

	timeZoneName isNil ifTrue: [timeZoneName := self class name].
	^ timeZoneName! !

LocalTimeTransform subclass: #VwOSTimeZone
	instanceVariableNames: 'timeZone'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Time-TimeZones'!

!VwOSTimeZone commentStamp: '<historical>' prior: 0!
I provide access to time zone functions provided by the base VisualWorks system.
!

!VwOSTimeZone class methodsFor: 'instance creation' stamp: ''!
here
	"Answer the current time zone, or nil if the classes have not been initialized for this system."

	"VwOSTimeZone here"

	^ super new initialize! !

!VwOSTimeZone methodsFor: 'initialize' stamp: 'dtl 1/8/2000 20:25'!
initialize
	"Be the proxy for the default TimeZone for this system image."

	self timeZone: (Smalltalk at: #TimeZone) default.
	self timeZoneName! !

!VwOSTimeZone methodsFor: 'leap seconds' stamp: ''!
leapSecondsAt: aPointInTime
	"Answer the number of leap seconds since January 1, 1970 UTC as of aPointInTime.
	Keep it simple, ignore leap seconds and just answer 0."

	^ 0! !

!VwOSTimeZone methodsFor: 'leap seconds' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
leapSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	"VwOSTimeZone here leapSecondsForDate: Date today time: Time now"

	| seconds t |
	seconds := aDate asSeconds + aTime asSeconds.
	t := PointInTime fromSmalltalkSeconds: seconds inTimeZone: self.
	^ self leapSecondsAt: t! !

!VwOSTimeZone methodsFor: 'offsets from UTC' stamp: ''!
localOffsetSecondsAt: aPointInTime
	"Answer the offset from UTC in this time zone as of aPointInTime"

	"VwOSTimeZone here localOffsetSecondsAt: PointInTime now"

	| sec |
	sec := aPointInTime asLocalSmalltalkSeconds.
	^ sec - (self timeZone convertLocalSecondsToGMT: (sec))! !

!VwOSTimeZone methodsFor: 'offsets from UTC' stamp: 'PaulDeBruicker 3/14/2016 08:28'!
localOffsetSecondsForDate: aDate time: aTime
	"Answer the offset from UTC in this time zone as of aDate and aTime, where
	aDate and aTime are in the context of this time zone."

	"VwOSTimeZone here localOffsetSecondsForDate: Date today time: Time now"

	| seconds t |
	seconds := aDate asSeconds + aTime asSeconds.
	t := PointInTime fromSmalltalkSeconds: seconds inTimeZone: self.
	^ self localOffsetSecondsAt: t! !

!VwOSTimeZone methodsFor: 'accessing' stamp: ''!
timeZone

	^ timeZone! !

!VwOSTimeZone methodsFor: 'accessing' stamp: ''!
timeZone: aTimeZone

	timeZone := aTimeZone! !

!VwOSTimeZone methodsFor: 'accessing' stamp: 'dtl 1/8/2000 20:29'!
timeZoneName
	"I do not know my name, but I am normally the system default time zone."

	timeZoneName isNil ifTrue: [timeZoneName := #default].
	^ timeZoneName! !
TimeZoneDstTransitionWatcher initialize!
TimeZoneProxy initialize!
PointInTime initialize!
PointInTimeNow initialize!
OSTimeZone initialize!
